from typing import Tuple
import warnings
from typing import Union

import numpy as np
import scipy.sparse as sp
from scipy.stats import zscore, percentileofscore

from Orange.data import Domain, Table
from Orange.preprocess.preprocess import Preprocess
from Orange.util import Enum
import Orange.statistics.util as ut


AnyArray = Union[np.ndarray, sp.csr_matrix, sp.csc_matrix]


class LogarithmicScale(Preprocess):
    Base = Enum("LogarithmicScale", ("BinaryLog", "NaturalLog", "CommonLog"),
                qualname="LogarithmicScale.Base")
    BinaryLog, NaturalLog, CommonLog = Base

    def __init__(self, base=BinaryLog):
        self.base = base

    def __call__(self, data: Table) -> Table:
        new_data = data.copy()

        if self.base == LogarithmicScale.BinaryLog:
            def func(x, *args, **kwargs):
                return np.log2(x + 1, *args, **kwargs)
        elif self.base == LogarithmicScale.CommonLog:
            def func(x, *args, **kwargs):
                return np.log10(x + 1, *args, **kwargs)
        elif self.base == LogarithmicScale.NaturalLog:
            func = np.log1p

        if sp.issparse(new_data.X):
            func(new_data.X.data, out=new_data.X.data)
        else:
            func(new_data.X, out=new_data.X)

        return new_data


class Binarize(Preprocess):
    Condition = Enum("Binarize", ("GreaterOrEqual", "Greater"),
                     qualname="Binarize.Condition")
    GreaterOrEqual, Greater = Condition

    def __init__(self, condition=GreaterOrEqual, threshold=1):
        self.condition = condition
        self.threshold = threshold

    def __call__(self, data: Table) -> Table:
        new_data = data.copy()
        if self.condition == Binarize.GreaterOrEqual:
            new_data.X = new_data.X >= self.threshold
        elif self.condition == Binarize.Greater:
            new_data.X = new_data.X > self.threshold
        return new_data


class Normalize(Preprocess):
    Method = Enum("Normalize", ("CPM", "Median"), qualname="Normalize.Method")
    CPM, Median = Method

    def __init__(self, method=CPM):
        self.method = method

    def __call__(self, *args):
        raise NotImplementedError

    def normalize(self, *args):
        raise NotImplementedError


class NormalizeSamples(Normalize):
    def __call__(self, data: Table) -> Table:
        new_data = data.copy()
        new_data.X = self.normalize(data.X)
        return new_data

    def normalize(self, table: AnyArray) -> AnyArray:
        row_sums = ut.nansum(table, axis=1)
        row_sums[row_sums == 0] = 1  # avoid division by zero errors

        if self.method == NormalizeSamples.Median:
            factor = np.nanmedian(row_sums)
        else:
            factor = 1e6

        if sp.issparse(table):
            table = sp.diags(1 / row_sums) @ table
        else:
            table = table / row_sums[:, None]

        table *= factor

        return table


class NormalizeGroups(Normalize):
    def __init__(self, group_var, method=Normalize.CPM):
        super().__init__(method)
        self.group_var = group_var

    def __call__(self, data: Table) -> Table:
        group_col = data.get_column_view(self.group_var)[0]
        group_col = group_col.astype("int64")
        new_data = data.copy()
        new_data.X = self.normalize(data.X, group_col)
        return new_data

    def normalize(self, table: AnyArray, group_col: np.ndarray) -> AnyArray:
        group_sums = np.bincount(group_col, ut.nansum(table, axis=1))
        group_sums[group_sums == 0] = 1
        group_sums_row = np.zeros_like(group_col)
        medians = []
        row_sums = ut.nansum(table, axis=1)
        for value, group_sum in zip(np.unique(group_col), group_sums):
            mask = group_col == value
            group_sums_row[mask] = group_sum
            if self.method == NormalizeGroups.Median:
                medians.append(np.nanmedian(row_sums[mask]))

        if self.method == NormalizeGroups.Median:
            factor = np.min(medians)
        else:
            factor = 1e6

        if sp.issparse(table):
            table = sp.diags(1 / group_sums_row) @ table
        else:
            table = table / group_sums_row[:, None]

        table *= factor

        return table


class Standardize(Preprocess):
    def __init__(self, lower_bound=None, upper_bound=None):
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound

    def __call__(self, data):
        new_data = data.copy()
        with np.errstate(invalid="ignore"):
            new_data.X = np.nan_to_num(zscore(data.X))
        if self.lower_bound is not None or self.upper_bound is not None:
            np.clip(new_data.X, self.lower_bound, self.upper_bound, new_data.X)
        return new_data


class SelectMostVariableGenes(Preprocess):
    Method = Enum("SelectMostVariableGenes",
                  ("Dispersion", "Variance", "Mean"),
                  qualname="SelectMostVariableGenes.Method")
    Dispersion, Variance, Mean = Method

    def __init__(self, method=Dispersion, n_genes=1000, n_groups=20):
        self.method = method
        self.n_genes = n_genes
        self.n_groups = n_groups if n_groups and n_groups > 1 else 1

    def __call__(self, data: Table) -> Table:
        n_groups = min(self.n_groups, len(data.domain.attributes))
        mean = ut.nanmean(data.X, axis=0)
        variance = ut.nanvar(data.X, axis=0)
        percentiles = [percentileofscore(mean, m) for m in mean]
        _, bins = np.histogram(percentiles, n_groups)
        bin_indices = np.digitize(percentiles, bins, True)
        # Right limit is treated differently in histogram and digitize
        # See https://github.com/numpy/numpy/issues/4217
        bin_indices[bin_indices == 0] = 1

        zscores = np.zeros_like(mean)
        for group in range(n_groups):
            group_indices, = np.where(bin_indices == group + 1)
            if self.method == SelectMostVariableGenes.Dispersion:
                group_mean = mean[group_indices]
                group_scores = np.divide(
                    variance[group_indices], group_mean,
                    out=np.zeros_like(group_mean), where=group_mean != 0
                )
            elif self.method == SelectMostVariableGenes.Variance:
                group_scores = variance[group_indices]
            elif self.method == SelectMostVariableGenes.Mean:
                group_scores = mean[group_indices]

            with np.errstate(invalid="ignore"):
                zscores[group_indices] = zscore(group_scores)

        indices = np.argsort(np.nan_to_num(zscores))[-self.n_genes:]
        return self._filter_columns(data, indices)

    @staticmethod
    def _filter_columns(data, indices):
        indices = sorted(indices)
        domain = data.domain
        attrs, cls, metas = domain.attributes, domain.class_vars, domain.metas
        domain = Domain(tuple(np.array(attrs)[indices]), cls, metas)
        return data.transform(domain)


class DropoutWarning(Warning):
    pass


class DropoutGeneSelection(Preprocess):
    def __init__(self, n_genes=None, decay=1, x_offset=5, y_offset=0.02,
                 threshold=0, at_least=0):
        self.n_genes = n_genes
        self.x_offset = x_offset
        self.y_offset = y_offset
        self.decay = decay
        self.threshold = threshold
        self.at_least = at_least

    def __call__(self, data: Table) -> Table:
        zero_rate, mean_expr = self.detection(data.X)
        selected = self.select_genes(zero_rate, mean_expr)
        if sum(selected) < self.n_genes:
            warnings.warn(f"{sum(selected)} genes selected", DropoutWarning)
        return self.filter_columns(data, selected)

    def detection(self, table: AnyArray) -> Tuple[np.ndarray, np.ndarray]:
        with np.errstate(invalid="ignore"):  # comparison can include nans
            mask = table > self.threshold

        if sp.issparse(table):
            A = table.copy()
            np.log2(A.data, out=A.data)
        else:
            A = np.ma.log2(table)  # avoid log2(0)
            A.mask = False

        detection_rate = ut.nanmean(mask, axis=0)
        zero_rate = 1 - detection_rate
        detected = detection_rate > 0
        detected_mean = ut.nanmean(A[:, detected], axis=0)

        mean_expr = np.full_like(zero_rate, fill_value=np.nan)
        mean_expr[detected] = detected_mean / detection_rate[detected]

        low_detection = np.array(np.sum(mask, axis=0)).squeeze()
        zero_rate[low_detection < self.at_least] = np.nan
        mean_expr[low_detection < self.at_least] = np.nan
        return zero_rate, mean_expr

    def select_genes(self, zero_rate: np.ndarray,
                     mean_expr: np.ndarray) -> np.ndarray:
        args = (mean_expr, zero_rate)
        return self.__get_selected(*args) if self.n_genes is None \
            else self.__bisection(*args)

    def __bisection(self, mean_expr, zero_rate):
        low, up = 0, 10
        for t in range(100):
            selected = self.__get_selected(mean_expr, zero_rate)
            if np.sum(selected) == self.n_genes:
                break
            elif np.sum(selected) < self.n_genes:
                up = self.x_offset
                self.x_offset = (self.x_offset + low) / 2
            else:
                low = self.x_offset
                self.x_offset = (self.x_offset + up) / 2
        return selected

    def __get_selected(self, mean_expr, zero_rate):
        nonan = ~np.isnan(zero_rate)
        sel = np.zeros_like(zero_rate).astype(bool)
        x = mean_expr[nonan]
        y = self.y(x, self.decay, self.x_offset, self.y_offset)
        sel[nonan] = (zero_rate[nonan] > y)
        return sel

    @staticmethod
    def y(x, decay, x_offset, y_offset):
        return np.exp(-decay * (x - x_offset)) + y_offset

    @staticmethod
    def filter_columns(data: Table, mask: np.ndarray) -> Table:
        domain = data.domain
        return data.transform(Domain(tuple(np.array(domain.attributes)[mask]),
                                     domain.class_vars,  domain.metas))



import unicodedata
from math import isnan, isinf, sqrt, ceil

from AnyQt.QtCore import Qt, QItemSelection, QItemSelectionModel
from AnyQt.QtGui import QStandardItem, QColor, QFont, QBrush, QPainter, QStandardItemModel
from AnyQt.QtWidgets import QTableView, QSizePolicy, QHeaderView, QStyledItemDelegate
from Orange.widgets import gui

BorderRole = next(gui.OrangeUserRole)
BorderColorRole = next(gui.OrangeUserRole)

CircleAreaRole = next(gui.OrangeUserRole)


class BorderedItemDelegate(QStyledItemDelegate):
    """Item delegate that paints border at the specified sides

    Data for `BorderRole` is a string containing letters t, r, b and/or l,
    which defines the sides at which the border is drawn.

    Role `BorderColorRole` sets the color for the cell. If not color is given,
    `self.color` is used as default.

    Args:
        color (QColor): default color (default default is black)
    """
    def __init__(self, color=Qt.black):
        super().__init__()
        self.color = color

    def paint(self, painter, option, index):
        """Overloads `paint` to draw borders"""
        QStyledItemDelegate.paint(self, painter, option, index)
        borders = index.data(BorderRole)
        if borders:
            color = index.data(BorderColorRole) or self.color
            painter.save()
            painter.setPen(color)
            rect = option.rect
            for side, p1, p2 in (("t", rect.topLeft(), rect.topRight()),
                                 ("r", rect.topRight(), rect.bottomRight()),
                                 ("b", rect.bottomLeft(), rect.bottomRight()),
                                 ("l", rect.topLeft(), rect.bottomLeft())):
                if side in borders:
                    painter.drawLine(p1, p2)
            painter.restore()


class CircleItemDelegate(BorderedItemDelegate, gui.VerticalItemDelegate):
    """
    Item delegate for display with circles. It switches between orienting
    text vertically, painting borders, and drawing circles depending
    on indices.

    Role `CircleAreaRole` should be a float between 0 and 1 (inclusive) and
    represents area of a circle.
    """
    def __init__(self, max_diameter, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.max_diameter = max_diameter

    def sizeHint(self, option, index):
        """
        Overloads sizeHint and provides sizeHint for vertical text, cell with
        borders, or circles depending on indices.
        """
        if index.column() == 0 or index.row() == 1:
            return gui.VerticalItemDelegate.sizeHint(self, option, index)
        elif index.column() == 1 or index.row() == 0:
            return BorderedItemDelegate.sizeHint(self, option, index)
        else:
            return QStyledItemDelegate.sizeHint(self, option, index)

    def paint(self, painter, option, index):
        """
        Overloads paint and switches between orienting text vertically,
        painting borders, and drawing circles depending on indices.
        """
        if index.column() == 0 or index.row() == 1:
            gui.VerticalItemDelegate.paint(self, painter, option, index)
        elif index.column() == 1 or index.row() == 0:
            BorderedItemDelegate.paint(self, painter, option, index)
        else:
            QStyledItemDelegate.paint(self, painter, option, index)
            area = index.data(CircleAreaRole)
            rect = option.rect
            radius = max(1, self.max_diameter/2*sqrt(area))
            painter.setPen(Qt.transparent)
            painter.setBrush(Qt.blue)
            painter.setRenderHint(QPainter.Antialiasing)
            painter.drawEllipse(rect.center(), radius, radius)


class ContingencyTable(QTableView):
    """
    A contingency table widget which can be used wherever ``QTableView`` could be used.

    Parameters
    ----------
    parent : Orange.widgets.widget.OWWidget
        The containing widget to which the table is connected.

    Attributes
    ----------
    classesv : :obj:`list` of :obj:`str`
        Vertical class headers.
    classesh : :obj:`list` of :obj:`str`
        Horizontal class headers.
    headerv : :obj:`str`, optional
        Vertical top header.
    headerh : :obj:`str`, optional
        Horizontal top header.
    corner_string : str
        String that is top right and bottom left corner of the table.
        Default is ``unicodedata.lookup("N-ARY SUMMATION")``.
    """

    def __init__(self, parent):
        super().__init__(editTriggers=QTableView.NoEditTriggers)

        self.bold_headers = None
        self.circles = False
        self.classesv = None
        self.classesh = None
        self.headerv = None
        self.headerh = None
        self.cleared = True
        self.parent = parent

        self.corner_string = unicodedata.lookup("N-ARY SUMMATION")

        self.tablemodel = QStandardItemModel(self)
        self.setModel(self.tablemodel)
        self.horizontalHeader().hide()
        self.verticalHeader().hide()
        self.setShowGrid(False)
        self.setSizePolicy(QSizePolicy.MinimumExpanding,
                           QSizePolicy.MinimumExpanding)
        self.clicked.connect(self._cell_clicked)

    def mouseReleaseEvent(self, e):
        super().mouseReleaseEvent(e)
        self.parent._invalidate()

    def keyPressEvent(self, event):
        super().keyPressEvent(event)
        self.parent._invalidate()

    def _cell_clicked(self, model_index):
        """Handle cell click event"""
        i, j = model_index.row(), model_index.column()
        if not i or not j:
            return
        n = self.tablemodel.rowCount()
        m = self.tablemodel.columnCount()
        index = self.tablemodel.index
        selection = None
        if i == j == 1 or not self.circles and i == n - 1 and j == m - 1:
            selection = QItemSelection(index(2, 2), index(n - 1, m - 1))
        elif i == 1 or not self.circles and i == n - 1:
            selection = QItemSelection(index(2, j), index(n - 1, j))
        elif j == 1 or not self.circles and j == m - 1:
            selection = QItemSelection(index(i, 2), index(i, m - 1))

        if selection is not None:
            self.selectionModel().select(
                selection, QItemSelectionModel.ClearAndSelect)

    def _item(self, i, j):
        return self.tablemodel.item(i, j) or QStandardItem()

    def _set_item(self, i, j, item):
        self.tablemodel.setItem(i, j, item)

    def set_variables(self, variablev, variableh, **kwargs):
        """
        Sets class headers and top headers and initializes table structure.

        Parameters
        ----------
        variablev : Orange.data.variable.DiscreteVariable
            Class headers are set to ``variablev.values``, top header is set to ``variablev.name``.
        variableh : Orange.data.variable.DiscreteVariable
            Class headers are set to ``variableh.values``, top header is set to ``variableh.name``.
        """
        self.classesv = variablev.values
        self.classesh = variableh.values
        self.headerv = variablev.name
        self.headerh = variableh.name
        self.initialize(**kwargs)

    def set_headers(self, classesv, classesh, headerv=None, headerh=None, **kwargs):
        """
        Sets class headers and top headers and initializes table structure.

        Parameters
        ----------
        classesv : :obj:`list` of :obj:`str`
            Vertical class headers.
        classesh : :obj:`list` of :obj:`str`
            Horizontal class headers.
        headerv : :obj:`str`, optional
            Vertical top header.
        headerh : :obj:`str`, optional
            Horizontal top header.
        """
        # numpy _str objects don't display properly
        self.classesv = list(map(str, classesv))
        self.classesh = list(map(str, classesh))
        self.headerv = headerv
        self.headerh = headerh
        self.initialize(**kwargs)

    def _style_cells(self):
        """
        Style all cells.
        """
        if self.circles:
            self.setItemDelegate(CircleItemDelegate(ceil(.9 * self.cell_size), Qt.white))
        else:
            self.setItemDelegate(BorderedItemDelegate(Qt.white))
        item = self._item(0, 2)
        item.setData(self.headerh, Qt.DisplayRole)
        item.setTextAlignment(Qt.AlignCenter)
        item.setFlags(Qt.NoItemFlags)

        self._set_item(0, 2, item)
        item = self._item(2, 0)
        item.setData(self.headerv, Qt.DisplayRole)
        item.setTextAlignment(Qt.AlignHCenter | Qt.AlignBottom)
        item.setFlags(Qt.NoItemFlags)
        self.setItemDelegateForColumn(0, gui.VerticalItemDelegate())
        self._set_item(2, 0, item)
        self.setSpan(0, 2, 1, len(self.classesh)+1)
        self.setSpan(2, 0, len(self.classesv)+1, 1)

        for i in (0, 1):
            for j in (0, 1):
                item = self._item(i, j)
                item.setFlags(Qt.NoItemFlags)
                self._set_item(i, j, item)

    def set_cell_size(self, cell_size):
        """
        Sets the size of table cells. The setting has an effect only if the the
        table is in circles display mode.

        Parameters
        ----------
        cell_size : int
            Size of a single table cell in pixels.
        """

        self.cell_size = cell_size
        if self.circles and not self.cleared:
            self.setItemDelegate(CircleItemDelegate(ceil(.9 * self.cell_size), Qt.white))
            self._resize()

    def _initialize_headers(self):
        """
        Fill headers with content and style them.
        """
        font = self.tablemodel.invisibleRootItem().font()
        bold_font = QFont(font)
        bold_font.setBold(True)

        for headers, ix in ((self.classesv + [self.corner_string], lambda p: (p + 2, 1)),
                            (self.classesh + [self.corner_string], lambda p: (1, p + 2))):
            for p, label in enumerate(headers):
                i, j = ix(p)
                item = self._item(i, j)
                item.setData(label, Qt.DisplayRole)
                if self.bold_headers:
                    item.setFont(bold_font)
                if not (i == 1 and self.circles):
                    item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
                item.setFlags(Qt.ItemIsEnabled)
                if p < len(headers) - 1:
                    item.setData("br"[j == 1], BorderRole)
                    item.setData(QColor(192, 192, 192), BorderColorRole)
                else:
                    item.setData("", BorderRole)
                self._set_item(i, j, item)

    def _resize(self):
        """
        Resize table to fit new contents and style.
        """
        if self.circles:
            self.verticalHeader().setDefaultSectionSize(self.cell_size)
            # If the following line is removed, VerticalItemDelegate.sizeHint()
            # of the headers returns sizes which are too short and the
            # headers are cut off.
            self.horizontalHeader().setDefaultSectionSize(100)
            self.resizeRowToContents(1)
            self.horizontalHeader().setDefaultSectionSize(self.cell_size)
            self.resizeColumnToContents(1)
            self.tablemodel.setRowCount(len(self.classesv) + 2)
            self.tablemodel.setColumnCount(len(self.classesh) + 2)
        else:
            if len(' '.join(self.classesh + [self.corner_string])) < 120:
                self.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
            else:
                self.horizontalHeader().setDefaultSectionSize(60)
            self.tablemodel.setRowCount(len(self.classesv) + 3)
            self.tablemodel.setColumnCount(len(self.classesh) + 3)

    def initialize(self, circles=False, cell_size=30, bold_headers=True):
        """
        Initializes table structure. Class headers must be set beforehand.

        Parameters
        ----------
        circles : :obj:`bool`, optional
            Turns on circle display. All table values should be between 0 and 1 (inclusive). Defaults to False.
        cell_size : :obj:`int`, optional
            Size of a single table cell. Ignored if circle display mode is off. Defaults to 30.
        bold_headers : :obj:`bool`, optional
            Whether the headers are bold or not. Defaults to True.
        """
        assert self.classesv is not None and self.classesh is not None

        self.circles = circles
        self.cell_size = cell_size
        self.bold_headers = bold_headers

        self._style_cells()
        self._initialize_headers()
        self._resize()

    def get_selection(self):
        """
        Get indexes of selected cells.

        Returns
        -------
        :obj:`set` of :obj:`tuple` of :obj:`int`
            Set of pairs of indexes.
        """
        return {(ind.row() - 2, ind.column() - 2) for ind in self.selectedIndexes()}

    def set_selection(self, indexes):
        """
        Set indexes of selected cells.

        Parameters
        ----------
        indexes : :obj:`set` of :obj:`tuple` of :obj:`int`
            Set of pairs of indexes.
        """
        selection = QItemSelection()
        index = self.model().index
        for row, col in indexes:
            sel = index(row + 2, col + 2)
            selection.select(sel, sel)
        self.selectionModel().select(
            selection, QItemSelectionModel.ClearAndSelect)

    def _set_sums(self, colsum, rowsum):
        """
        Set content of cells on bottom and right edge.

        Parameters
        ----------
        colsum : numpy.array
            Content of cells on bottom edge.
        rowsum : numpy.array
            Content of cells on right edge.
        """
        bold_font = self.tablemodel.invisibleRootItem().font()
        bold_font.setBold(True)

        def _sum_item(value, border=""):
            item = QStandardItem()
            item.setData(value, Qt.DisplayRole)
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            item.setFlags(Qt.ItemIsEnabled)
            item.setFont(bold_font)
            item.setData(border, BorderRole)
            item.setData(QColor(192, 192, 192), BorderColorRole)
            return item

        for i in range(len(self.classesh)):
            self._set_item(len(self.classesv) + 2, i + 2, _sum_item(int(colsum[i]), "t"))
        for i in range(len(self.classesv)):
            self._set_item(i + 2, len(self.classesh) + 2, _sum_item(int(rowsum[i]), "l"))
        self._set_item(len(self.classesv) + 2, len(self.classesh) + 2, _sum_item(int(rowsum.sum())))

    def _set_values(self, matrix, colors, formatstr, tooltip):
        """
        Set content of cells which aren't headers and don't represent aggregate values.

        Parameters
        ----------
        matrix : numpy.array
            2D array to be set as data.
        colors : :obj:`numpy.array`
            2D array with color values.
        formatstr : :obj:`str`, optional
            Format string for cell data.
        tooltip : :obj:`(int, int) -> str`
            Function which takes vertical index and horizontal index as arguments and returns
            desired tooltip as a string.
        """
        def _isinvalid(x):
            return isnan(x) or isinf(x)

        for i in range(len(self.classesv)):
            for j in range(len(self.classesh)):
                val = matrix[i, j]
                col_val = float('nan') if colors is None else colors[i, j]
                item = QStandardItem()
                if self.circles:
                    item.setData(val, CircleAreaRole)
                else:
                    item.setData(
                        "NA" if _isinvalid(val) else formatstr.format(val),
                        Qt.DisplayRole)
                    bkcolor = QColor.fromHsl(
                        [0, 240][i == j], 160,
                        255 if _isinvalid(col_val) else int(255 - 30 * col_val))
                    item.setData(QBrush(bkcolor), Qt.BackgroundRole)
                item.setData("trbl", BorderRole)
                if tooltip is not None:
                    item.setToolTip(tooltip(i, j))
                item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
                item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
                self._set_item(i + 2, j + 2, item)

    def update_table(self, matrix, colsum=None, rowsum=None, colors=None, formatstr="{}", tooltip=None):
        """
        Sets ``matrix`` as data of the table.

        Parameters
        ----------
        matrix : numpy.array
            2D array to be set as data.
        colsum : :obj:`numpy.array`, optional
            1D optional array with aggregate values of columns, defaults to sum.
        rowsum : :obj:`numpy.array`, optional
            1D optional array with aggregate values of rows, defaults to sum.
        colors : :obj:`numpy.array`, optional
            2D array with color values, defaults to no color.
        formatstr : :obj:`str`, optional
            Format string for cell data, defaults to ``"{}"``.
        tooltip : :obj:`(int, int) -> str`, optional
            Function which takes vertical index and horizontal index as arguments and returns
            desired tooltip as a string. Defaults to no tooltips.
        """
        self.cleared = False
        selected_indexes = self.get_selection()

        self._set_values(matrix, colors, formatstr, tooltip)
        if not self.circles:
            if colsum is None:
                colsum = matrix.sum(axis=0)
            if rowsum is None:
                rowsum = matrix.sum(axis=1)
            self._set_sums(colsum, rowsum)

        self.set_selection(selected_indexes)

    def clear(self):
        """
        Clears the table.
        """
        self.tablemodel.clear()
        self.cleared = True



import Orange
import numpy as np
import time
from functools import lru_cache

from orangecontrib.bioinformatics.widgets.utils.data import GENE_ID_ATTRIBUTE
from scipy.stats import hypergeom
from sklearn.cluster import SpectralBiclustering
from Orange.data import Domain, DiscreteVariable, ContinuousVariable, Table


class ClusterAnalysis:
    """
    Analysis of single cell clusters based on enriched genes.

    Parameters
    ----------
    data : Orange table of cells with genes as attributes and each row has a defined cluster
    cluster_var : string of preferred row class_var


    Attributes
    ----------
    data : Orange table of cells with genes as attributes and each cell belongs to a cluster
    clusters_rows : list of indexes in which cluster belongs each cell
    clusters_ind : list of unique indexes that appear in data
    clusters_names : ordered list of all cluster names that appear in data
    genes : list of genes for which we want the output
    model : result table of percentage expressing
    o_model : result Orange.data.Table of percentage expressing
    row_order_ : order of clusters after spectral biclustering
    column_order_ : order of genes after spectral biclustering
    enriched_matrix_low : percentages for under-expression of genes
    enriched_matrix_high : percentages for over-expression of genes
    enriched_matrix : percentages for gene expression according to given 'enrichment' parameter
    enrichment : set of available enrichment parameters
    """

    def __init__(self, data, cluster_var='Cluster', callback=None):
        self.data = data
        self.gene_id_attribute = self.data.attributes.get(GENE_ID_ATTRIBUTE, None)

        self.X = self.data.X > 0

        if cluster_var is not None:
            self.class_var = self.data.domain[cluster_var]
        else:
            self.class_var = self.data.domain.class_var

        self.clusters_rows = self.data.get_column_view(self.class_var)[0]
        self.clusters_names = self.class_var.values
        self.clusters_ind = np.unique(self.clusters_rows)

        # take only those that appear in data
        self.clusters_names = [self.clusters_names[int(i)] for i in self.clusters_ind]
        self.columns = self.data.domain.attributes

        self.enriched_matrix = None
        self.enrichment = {'high', 'low', 'either'}
        self.enriched_matrix_low = None
        self.enriched_matrix_high = None
        self.model = None
        self.o_model = None
        self.row_order_ = None
        self.column_order_ = None
        self.genes = None

        self._create_enriched_matrix(callback=callback)

    def _create_enriched_matrix(self, callback=None):
        """
        Create matrix of how much each gene enriches each cluster.
        shape: (clusters_names, genes)
        """
        # get a list of indexes of all genes (NOTE: can cause bugs)
        genes = range(len(self.columns))

        Z = self.X

        # n, M for hypergeometric distribution
        count_all_pos = np.sum(Z, axis=0)  # n - all positive (number of success states in the population)
        count_all = Z.shape[0]  # M - all (population size)

        low = np.empty(shape=(len(self.clusters_names), len(genes)), dtype='float64')
        high = np.empty(shape=(len(self.clusters_names), len(genes)), dtype='float64')

        # for each cluster we calculate the n_enriched most enriched
        for c in range(len(self.clusters_names)):
            if callback is not None:
                callback(c / len(self.clusters_names))
            # get all cells that belong to cluster c
            cells = Z[self.clusters_rows == self.clusters_ind[c]]

            # k, N for hypergeometric distribution
            count_cluster_pos = np.sum(cells, axis=0)  # k - positive in cluster c ) number of observed successes)
            count_cluster = cells.shape[0]  # N - all in cluster c (number of draws)

            # calculate cdf for every gene
            low[c] = np.array(
                [hypergeom.cdf(count_cluster_pos[i], count_all, count_all_pos[i], count_cluster) for i in
                 genes])
            high[c] = hypergeom.sf(count_cluster_pos, count_all, count_all_pos, count_cluster) + \
                      hypergeom.pmf(count_cluster_pos, count_all, count_all_pos, count_cluster)

        self.enriched_matrix_low = low
        self.enriched_matrix_high = high

    def intersection(self, gene_list):
        """
        Get intersection of genes in the dataset and in the iterable.

        Parameters
        ----------
        gene_list : iterable
            Genes to find in the dataset.

        Returns
        -------
        list
            List of genes in the intersection.
        """
        gene_set = set()

        if gene_list:
            gene_set = set([str(gene) for gene in gene_list])

        return [gene.attributes[self.gene_id_attribute] for gene in self.columns
                if self.gene_id_attribute in gene.attributes
                and str(gene.attributes[self.gene_id_attribute]) in gene_set]

    @lru_cache(maxsize=3)
    def enriched_genes(self, gene_list, enrichment=None, biclustering=True, callback=None):
        """
        Cluster-based enrichment scores for a tuple of genes

        Parameters
        ----------
        gene_list : tuple of ints, gene ids
        enrichment : ignored, used for consistency in function call
        biclustering : boolean, return biclustering model
        """
        # fix enrichment at 'either', to calculate the shortest tail p-value
        enrichment = 'either'
        self.enriched_matrix = np.hstack((self.enriched_matrix_low, self.enriched_matrix_high))
        enumerated_attributes = enumerate(gene.attributes[self.gene_id_attribute] for gene in self.columns
                                          if self.gene_id_attribute in gene.attributes)
        self.genes = [attribute[0] for attribute in enumerated_attributes if attribute[1] in gene_list]

        return self._create_model(enrichment, biclustering, callback=callback)

    @lru_cache(maxsize=3)
    def enriched_genes_per_cluster(self, n=3, enrichment='high', biclustering=True, callback=None):
        """
        n genes that are most enriched for each cluster.

        Parameters
        ----------
        n : int, number of enriched genes per cluster
        enrichment : string, type of enrichment (high, low, either)
        biclustering : boolean, return biclustering model
        """
        res = list()

        # get a list of indexes of all genes
        genes = list(range(len(self.columns)))
        genes_enum = genes

        # handle enrichment matrix according to enrichment parameter
        if enrichment not in self.enrichment:
            raise ValueError("enrichment should be either 'high', 'low' or 'either'"
                             ", but a value %r was passed" %
                             enrichment)
        elif enrichment == 'high':
            self.enriched_matrix = self.enriched_matrix_high
        elif enrichment == 'low':
            self.enriched_matrix = self.enriched_matrix_low
        elif enrichment == 'either':
            self.enriched_matrix = np.hstack((self.enriched_matrix_low, self.enriched_matrix_high))
            genes.extend(genes)
            genes_enum = list(range(len(genes)))

        # for each cluster we calculate the n most enriched genes
        for c, enriched_percent in enumerate(self.enriched_matrix):
            # search amongst the genes that haven't bee selected yet
            enriched_percent = enriched_percent[genes_enum]

            # sorting
            zipped = list(zip(*sorted(zip(enriched_percent, genes), key=lambda x: x[0])))
            enriched_genes = list(zipped[1][:n])

            # remove found enriched genes from list of all genes (ensure every cluster has n unique enriched genes)
            to_remove_enum = enriched_genes + [g + len(self.columns) for g in enriched_genes]
            genes = [g for g in genes if g not in enriched_genes]
            genes_enum = [g for g in genes_enum if g not in to_remove_enum]

            res.extend(enriched_genes)

        self.genes = res
        return self._create_model(enrichment, biclustering, callback=callback)

    @lru_cache(maxsize=3)
    def enriched_genes_data(self, n=20, enrichment='high', biclustering=True, callback=None):
        """
        n top enriched genes, where "top" means for any cluster

        Parameters
        ----------
        n : int, number of enriched genes overall
        enrichment : string, type of enrichment (high, low, either)
        biclustering : boolean, return biclustering model
        """
        res = list()

        # get a list of indexes of all genes
        genes = list(range(len(self.columns)))

        # handle enrichment matrix according to enrichment parameter
        if enrichment not in self.enrichment:
            raise ValueError("enrichment should be either 'high', 'low' or 'either'"
                             ", but a value %r was passed" %
                             enrichment)
        elif enrichment == 'high':
            self.enriched_matrix = self.enriched_matrix_high
        elif enrichment == 'low':
            self.enriched_matrix = self.enriched_matrix_low
        elif enrichment == 'either':
            self.enriched_matrix = np.hstack((self.enriched_matrix_low, self.enriched_matrix_high))
            genes.extend(genes)

        # select n enriched genes for each cluster
        for enriched_percent in self.enriched_matrix:
            zipped = sorted(zip(enriched_percent, genes), key=lambda x: x[0])
            enriched_genes_tuple = zipped[:n]

            res.extend(enriched_genes_tuple)

        # sort whole list
        res = sorted(res, key=lambda x: x[0])

        # remove duplicate genes that are less enriched
        seen = set()
        res = [item for item in res if item[1] not in seen and not seen.add(item)]
        res = list(zip(*res))

        # take only n genes
        self.genes = res[1][:n]

        return self._create_model(enrichment, biclustering, callback=callback)

    def _fraction_expressing(self, enrichment, callback=None):
        """
        Expression percentage and p-values for each enriched gene for each cluster.

        Params
        ----------
        enrichment : string, used internally for p-value
        """
        Z = self.X
        res = list()
        pvalues = list()

        for c in range(len(self.clusters_names)):
            if callback is not None:
                callback(c/len(self.clusters_names) * 0.2)
            cells = Z[self.clusters_rows == self.clusters_ind[c]]
            # calculate fraction expressing
            res.append([sum(cells[:, gene] > 0) / cells.shape[0] if cells.shape[0] > 0 else 0 for gene in self.genes])

            # if 'either' - we choose the lower p-value one (higher wouldn't have been chosen beforehand)
            if enrichment == 'either':
                pvalues.append([min(self.enriched_matrix[c, gene],
                                    self.enriched_matrix[c, gene + len(self.columns)]) for gene in
                                self.genes])
            # else we take p-value normally
            else:
                pvalues.append([self.enriched_matrix[c, gene] for gene in self.genes])

        self.model = np.array(res)
        self.pvalues = np.array(pvalues)

    @staticmethod
    def biclustering(matrix, distance, callback=None):
        if min(matrix.shape) <= 2:
            return np.arange(matrix.shape[0]), np.arange(matrix.shape[1])

        best_score = np.iinfo(np.dtype('uint16')).max
        best_model = None

        # find the best biclusters (needs revision)
        limit = int(min(matrix.shape) / 2) - 1
        limit = 3 if limit < 3 else limit
        for i in range(2, limit):
            if callback is not None:
                callback(0.2 + (i - 2) / (limit - 2) * 0.8)
            # perform biclustering
            model = SpectralBiclustering(
                n_clusters=i, method='log', random_state=0)
            model.fit(matrix)
            fit_data = matrix[np.argsort(model.row_labels_)]
            fit_data = fit_data[:, np.argsort(model.column_labels_)]

            # calculate score and save the lowest one
            score = distance(fit_data)
            if score < best_score:
                best_score = score
                best_model = model

        return np.argsort(best_model.row_labels_), np.argsort(best_model.column_labels_)

    @staticmethod
    def reorder(matrix, row_order, column_order):
        return

    def _sort_fraction_expressing(self, callback=None):
        """
        Sort rows and columns based on expressed percentages.
        """

        self.row_order_, self.column_order_ = self.biclustering(self.model,
                                                                self.neighbor_distance,
                                                                callback)

        self.model = self.model[self.row_order_]
        self.model = self.model[:, self.column_order_]

        # order p-values matrix too
        self.pvalues = self.pvalues[self.row_order_]
        self.pvalues = self.pvalues[:, self.column_order_]

    def _create_model(self, enrichment, biclustering, callback):
        """
        Create cluster analysis model.

        Params
        ----------
        enrichment : string, used internally

        Return
        ----------
        res_rows : list of strings, clusters_names
        res_genes : list of strings, genes
        fraction_expressing : numpy matrix, fraction of expressing for enriched genes per cluster
        p-value : numpy matrix, p-values for enriched genes for each cluster
        """

        self._fraction_expressing(enrichment)

        if not biclustering:
            self.row_order_ = list(range(len(self.clusters_names)))
            self.column_order_ = list(range(len(self.genes)))
        else:
            self._sort_fraction_expressing(callback=callback)

        res_genes = list(np.ravel([self.columns[self.genes[i]].name for i in self.column_order_]))
        res_rows = [self.clusters_names[i] for i in self.row_order_]
        return res_rows, res_genes, self.model, self.pvalues

    @staticmethod
    def neighbor_distance(X):
        """
        Calculate euclicean distance between neighbors in rows and columns

        Params
        ----------
        X : (n_clusters, n_genes) : biclustered matrix of expressing percentage

        Return
        ----------
        res : sum of distances
        """
        # rows
        res = np.sum([np.linalg.norm(X[i - 1] - X[i]) for i in range(1, len(X))])
        # columns
        res += np.sum([np.linalg.norm(X[:, i - 1] - X[:, i]) for i in range(1, len(X[0]))])
        return res

    def draw_pyplot(self):
        """
        Draw with pyplot clusters_names on y and enriched genes on x, with size of circle representing
        percentage of expressed.
        """
        import matplotlib.pyplot as plt

        circle_mult = .4
        x_len = range(len(self.column_order_))
        y_len = range(len(self.row_order_))

        ## Plot
        fig, ax = plt.subplots()

        for i in range(len(self.model)):
            for j in range(len(self.model[0])):
                ax.add_artist(plt.Circle((j, i), self.model[i][j] * circle_mult))

        plt.xlim(-1, len(self.column_order_))
        plt.ylim(-1, len(self.row_order_))

        ## Y axis - clusters_names
        clusters_list = self.clusters_names
        clusters = [clusters_list[i] for i in self.row_order_]
        plt.yticks(y_len, clusters, rotation='horizontal')
        plt.margins(0.2)

        ## X axis - genes
        genes_list = self.genes
        genes = np.ravel(list([self.columns[genes_list[i]].name for i in self.column_order_]))

        plt.xticks(x_len, genes, rotation='vertical')
        plt.margins(0.2)

        plt.subplots_adjust(bottom=0.2)
        # plt.savefig('fig1.png')
        plt.show()
        return plt

    @staticmethod
    def contingency_table(data: np.array, rows: DiscreteVariable, columns: list, metas) -> Table:
        return Table.from_numpy(Domain([ContinuousVariable.make(column) for column in columns], metas=[rows]),
                                data, metas=metas)

    def create_contingency_table(self):
        """
        Create Orange.table from results

        Return
        --------
        o_model : Orange.Table
        """
        # create Orange.Table for calculated model
        dmn = [self.columns[self.genes[i]].name for i in self.column_order_]
        mts = DiscreteVariable.make(self.class_var.name, values=self.clusters_names)
        self.o_model = self.contingency_table(self.model, mts, dmn, np.array([self.row_order_]).T)
        return self.o_model


if __name__ == '__main__':
    # Example usages

    # data = Orange.data.Table('data/bone_marrow_louvain.pickle')
    data = Orange.data.Table('data/bone_marrow_louvain.pickle')

    start = time.time()
    print("START")
    CA = ClusterAnalysis(data, cluster_var='Cluster')
    print(time.time() - start)
    res = CA.enriched_genes(gene_list=('HBG1', 'S100A9', 'HBG2'))
    print(time.time() - start)
    CA.draw_pyplot()

    res = CA.enriched_genes_per_cluster(n=2, enrichment='either', biclustering=False)
    print(time.time() - start)
    CA.draw_pyplot()

    res = CA.enriched_genes_data(enrichment='low')
    print(time.time() - start)
    CA.draw_pyplot()



import numpy as np
from AnyQt.QtCore import QSize
from sklearn.preprocessing import scale

from Orange.data import ContinuousVariable, Domain, Table
from Orange.statistics.util import nanmax, nanmean, nanmedian
from Orange.widgets import widget, gui
from Orange.widgets.settings import Setting
from Orange.widgets.utils.signals import Output, Input
from Orange.widgets.utils.sql import check_sql_input
from Orange.widgets.widget import OWWidget, Msg

from orangecontrib.bioinformatics.widgets.utils.data import (
    GENE_ID_ATTRIBUTE, GENE_ID_COLUMN, GENE_AS_ATTRIBUTE_NAME, TAX_ID
)


def percent_nonzero(x, axis=1):
    return np.ravel(np.mean(x > 0, axis=axis))


def mean_normalized(x, axis=1):
    return np.mean(scale(x, with_mean=False), axis=axis)


class OWScoreCells(widget.OWWidget):
    name = "Score Cells"
    description = "Add a cell score based on the given set of genes"
    icon = "icons/ScoreCells.svg"
    priority = 320

    auto_apply = Setting(True)
    aggregation = Setting('Mean expression')
    score_variable_name = Setting('Score', schema_only=True)
    want_main_area = False

    class Warning(OWWidget.Warning):
        no_genes = Msg("No matching genes in data")
        some_genes = Msg("{} (of {}) genes not found in data")

    class Error(OWWidget.Error):
        organism_mismatch = Msg(
            'Organism in input data and marker genes does not match')

        # for input data
        missing_annotation_input = Msg(
            'Missing annotation on gene IDs and organism in the input genes data.')
        missing_gene_id_input = Msg(
            'Gene identification info missing in input genes data.')
        missin_organism_input = Msg(
            'Missing organism information in the input genes data')

        # for marker data
        missing_annotation_marker = Msg(
            'Missing annotation on gene IDs and organism in the marker genes data.')
        missing_gene_id_marker = Msg(
            'Gene identification info missing in marker genes data.')
        missin_organism_marker = Msg(
            'Missing organism information in the marker genes data')

    class Inputs:
        data = Input("Data", Table)
        genes = Input("Genes", Table)

    class Outputs:
        data = Output("Data", Table)

    aggregation_functions = {
        'Mean expression': nanmean,
        'Mean normalized expression': mean_normalized,
        'Median expression': nanmedian,
        'Maximum expression': nanmax,
        'Fraction of expressed markers': percent_nonzero,
    }

    def __init__(self):
        super().__init__()

        # Input data attributes
        self.input_data = None
        self.input_genes = None
        self.tax_id = None
        self.use_attr_names = None
        self.gene_id_attribute = None
        self.gene_id_column = None

        # Marker data attributes
        self.marker_data = None
        self.marker_genes = None
        self.marker_tax_id = None
        self.marker_use_attr_names = None
        self.marker_gene_id_attribute = None
        self.marker_gene_id_column = None

        info_box = gui.vBox(self.controlArea, 'Input info')
        self.info_text = gui.widgetLabel(info_box)

        box = gui.vBox(self.controlArea, "Aggregation")
        gui.comboBox(box, self, 'aggregation', sendSelectedValue=True,
                     items=list(self.aggregation_functions.keys()),
                     callback=lambda: self.commit())

        box = gui.vBox(self.controlArea, "Score Column Name in Output Data: ")
        self.line_edit = gui.lineEdit(box, self, 'score_variable_name',
                                      callback=lambda: self.commit())
        self.line_edit.setPlaceholderText('Column Name ...')

        self.apply_button = gui.auto_commit(
            self.controlArea, self, "auto_apply", "&Apply", box=False)

        self.sizeHint()

    def sizeHint(self):
        return QSize(320, 240)

    def handleNewSignals(self):
        self.update_info_box()
        self.commit()
        
    def __check_organism_mismatch(self):
        """ Check if organisms from different inputs match.

        :return: True if there is a mismatch
        """
        if self.tax_id and self.marker_tax_id:
            return self.tax_id != self.marker_tax_id
        return False

    def __clear_widget_state(self):
        self.Warning.clear()
        self.Error.clear()

    @Inputs.data
    @check_sql_input
    def set_data(self, data):
        self.__clear_widget_state()
        self.input_genes = None
        self.input_data = data

        if self.input_data:
            self.tax_id = str(self.input_data.attributes.get(TAX_ID, None))
            self.use_attr_names = self.input_data.attributes.get(GENE_AS_ATTRIBUTE_NAME, None)
            self.gene_id_attribute = self.input_data.attributes.get(GENE_ID_ATTRIBUTE, None)
            self.gene_id_column = self.input_data.attributes.get(GENE_ID_COLUMN, None)

            if not (self.use_attr_names is not None
                    and ((self.gene_id_attribute is None) ^ (self.gene_id_column is None))):

                if self.tax_id is None:
                    self.Error.missing_annotation_input()
                    return

                self.Error.missing_gene_id_input()
                return

            elif self.tax_id is None:
                self.Error.missin_organism_input()
                return

            if self.__check_organism_mismatch():
                self.Error.organism_mismatch()
                return

            self.__get_input_genes()

    @Inputs.genes
    @check_sql_input
    def set_genes(self, data):
        self.__clear_widget_state()
        self.marker_genes = None
        self.marker_data = data

        if self.marker_data:
            self.marker_tax_id = str(self.marker_data.attributes.get(TAX_ID, None))
            self.marker_use_attr_names = self.marker_data.attributes.get(GENE_AS_ATTRIBUTE_NAME, None)
            self.marker_gene_id_attribute = self.marker_data.attributes.get(GENE_ID_ATTRIBUTE, None)
            self.marker_gene_id_column = self.marker_data.attributes.get(GENE_ID_COLUMN, None)

            if not (self.marker_use_attr_names is not None
                    and ((self.marker_gene_id_attribute is None) ^ (self.marker_gene_id_column is None))):

                if self.marker_tax_id is None:
                    self.Error.missing_annotation_marker()
                    return

                self.Error.missing_gene_id_marker()
                return

            elif self.marker_tax_id is None:
                self.Error.missin_organism_marker()
                return

            if self.__check_organism_mismatch():
                self.Error.organism_mismatch()
                return

            self.__get_marker_genes()

    def __score_cells(self):
        scores = np.ones(len(self.input_data))

        if self.marker_data and self.marker_genes and \
                self.input_data and self.input_genes:

            matched_ids = [gene_id for gene_id in self.input_genes if gene_id in self.marker_genes]
            matched_columns = [column for column in self.input_data.domain.attributes
                               if self.gene_id_attribute in column.attributes and
                               str(column.attributes[self.gene_id_attribute]) in matched_ids]

            self.Warning.no_genes.clear()
            self.Warning.some_genes.clear()

            if not matched_ids:
                self.Warning.no_genes()
            else:
                self.update_info_box(matched_genes=matched_ids)
                if len(matched_ids) < len(self.marker_genes):
                    self.Warning.some_genes(len(self.marker_genes) - len(matched_ids),
                                            len(self.marker_genes))

                values = self.input_data[:, matched_columns].X
                aggregator = self.aggregation_functions[self.aggregation]
                scores = aggregator(values, axis=1)

        return scores

    def __get_input_genes(self):
        self.input_genes = []

        if self.use_attr_names:
            for variable in self.input_data.domain.attributes:
                self.input_genes.append(str(variable.attributes.get(self.gene_id_attribute, '?')))
        else:
            genes, _ = self.input_data.get_column_view(self.gene_id_column)
            self.input_genes = [str(g) for g in genes]

        self.input_genes = set(filter(None, self.input_genes))

    def __get_marker_genes(self):
        self.marker_genes = []

        if self.marker_use_attr_names:
            for variable in self.marker_data.domain.attributes:
                self.marker_genes.append(str(variable.attributes.get(self.marker_gene_id_attribute, '?')))
        else:
            genes, _ = self.marker_data.get_column_view(self.marker_gene_id_column)
            self.marker_genes = [str(g) for g in genes]

        self.marker_genes = set(filter(None, self.marker_genes))

    def commit(self):
        table = None

        if self.input_data:
            score = self.__score_cells()

            score_var = ContinuousVariable(self.score_variable_name)
            domain = Domain(self.input_data.domain.attributes,
                            self.input_data.domain.class_vars,
                            self.input_data.domain.metas + (score_var,))

            table = self.input_data.transform(domain)
            col, sparse = table.get_column_view(score_var)
            col[:] = score

        self.Outputs.data.send(table)

    def send_report(self):
        gene = None
        if self.marker_data is not None:
            gene = self.gene
            if gene in self.marker_data.domain:
                gene = self.marker_data.domain[gene]
        self.report_items((
            ("Gene", gene),
        ))

    def update_info_box(self, matched_genes=None):
        info_string = ''
        if self.input_data and self.input_genes:
            info_string += 'Data: {} cells, {} genes.\n'.format(len(self.input_data), len(self.input_genes))

        if self.marker_genes:
            info_string += 'Markers: {} genes.\n'.format(len(self.marker_genes))

        if not info_string:
            info_string = 'No genes on input.\n'

        if matched_genes:
            info_string += 'Gene matching: {} markers found in input data.\n'.format(len(matched_genes))

        self.info_text.setText(info_string)


if __name__ == "__main__":

    def test():
        from AnyQt.QtWidgets import QApplication
        app = QApplication([])

        w = OWScoreCells()
        data = Table("iris")
        w.set_data(data)
        w.handleNewSignals()
        w.show()
        app.exec_()

    test()



import numbers
import sys
from collections import namedtuple
from enum import IntEnum
import numpy as np

from AnyQt.QtCore import Qt
from AnyQt.QtGui import QStandardItemModel, QStandardItem
from AnyQt.QtWidgets import (
    QStyledItemDelegate, QTreeView, QHeaderView, QApplication
)

from Orange.data import Table
from Orange.widgets import gui
from Orange.widgets.settings import (
    ContextSetting, Setting, PerfectDomainContextHandler
)
from Orange.widgets.widget import Input, Output, Msg, OWWidget

from orangecontrib.single_cell.preprocess.scbnorm import (
    ScBatchScorer, SCBatchNormalizer, LINKS
)

VariableRole = next(gui.OrangeUserRole)


class IntegralDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, numbers.Number):
            option.displayAlignment = Qt.AlignRight | Qt.AlignVCenter


class RealDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, numbers.Number):
            option.displayAlignment = Qt.AlignRight | Qt.AlignVCenter
            option.text = "?" if np.isnan(data) else "{:.3f}".format(data)


class LinkMethod(IntEnum):
    IDENTITY_LINK, LOG_LINK = range(2)

    @staticmethod
    def items():
        return sorted(list(LINKS.keys()))


class OWBatchNorm(OWWidget):
    name = "Batch Effect Removal"
    description = "Batch effect normalization on Single Cell data set."
    icon = "icons/BatchEffectRemoval.svg"
    priority = 230

    class Inputs:
        data = Input("Data", Table)

    class Outputs:
        data = Output("Data", Table)

    class Error(OWWidget.Error):
        general_error = Msg({})
        discrete_attributes = Msg("Data with discrete attributes "
                                  "can not be processed.")

    class Warning(OWWidget.Warning):
        missing_values = Msg("Missing values have been replaced with 0.")
        negative_values = Msg("Unable to use current settings due "
                              "to negative values in data.")

    resizing_enabled = False
    want_main_area = False

    settingsHandler = PerfectDomainContextHandler()
    batch_vars = ContextSetting([])
    link_method = Setting(LinkMethod.IDENTITY_LINK)
    skip_zeros = Setting(False)
    auto_commit = Setting(True)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.data = None

        # Info
        infobox = gui.widgetBox(self.controlArea, "Info")
        self.info_label = gui.widgetLabel(infobox, "No data on input.")

        # Link method
        method_box = gui.widgetBox(self.controlArea, "Method")
        gui.comboBox(
            method_box, self, "link_method", items=LinkMethod.items(),
            callback=self.__link_method_changed
        )
        gui.separator(method_box)
        self.skip_zeros_check = gui.checkBox(
            method_box, self, "skip_zeros", "Skip zero expressions",
            enabled=self.link_method != LinkMethod.LOG_LINK,
            callback=lambda: self.commit()
        )

        # Batch Variable Selection
        header_shema = (
            ("selected", ""),
            ("variable", "Variable"),
            ("count", "#"),
            ("score", "Score")
        )
        header_labels = labels = [label for _, label in header_shema]
        header = namedtuple("header", [tag for tag, _ in header_shema])
        self.Header = header(*[index for index, _ in enumerate(labels)])

        batch_box = gui.widgetBox(self.controlArea, "Batch Variable Selection")
        self.view = QTreeView()
        self.model = QStandardItemModel()
        self.model.itemChanged.connect(self.__selected_batch_vars_changed)
        self.model.setHorizontalHeaderLabels(header_labels)
        batch_box.layout().addWidget(self.view)
        self._setup_view()

        gui.auto_commit(self.controlArea, self, "auto_commit",
                        "Apply", "Apply Automatically")

    def __link_method_changed(self):
        enable = self.link_method != LinkMethod.LOG_LINK
        self.skip_zeros_check.setEnabled(enable)
        if not enable:
            self.skip_zeros_check.setChecked(True)
        self.commit()

    def __selected_batch_vars_changed(self, item):
        if item.checkState():
            self.batch_vars.append(item.data(VariableRole))
        else:
            self.batch_vars.remove(item.data(VariableRole))
        self.commit()

    def _setup_view(self):
        self.view.setModel(self.model)
        self.view.setSelectionMode(QTreeView.NoSelection)
        self.view.setSortingEnabled(True)
        self.view.setRootIsDecorated(False)
        self.view.setItemDelegateForColumn(self.Header.count,
                                           IntegralDelegate(self))
        self.view.setItemDelegateForColumn(self.Header.score,
                                           RealDelegate(self))
        self.view.header().setSectionResizeMode(QHeaderView.ResizeToContents)
        self.view.header().setStretchLastSection(False)
        self.view.header().setSectionResizeMode(self.Header.variable,
                                                QHeaderView.Stretch)
        self.view.setFocus()

    @Inputs.data
    def set_data(self, data):
        self.closeContext()
        self.clear()
        self.data = data
        self._setup_info_label()
        self._check_data()
        self.openContext(data)
        if self.data is not None:
            self.batch_vars = [data.domain[v.name] for v in self.batch_vars]
            self._setup_model()
        self.commit()

    def clear(self):
        self.batch_vars = []
        if self.model:
            n_rows = self.model.rowCount()
            self.model.removeRows(0, n_rows)

    def _setup_info_label(self):
        text = "No data on input."
        if self.data is not None:
            domain, attrs = self.data.domain, self.data.domain.attributes
            text = "{} cells, {} genes\n".format(len(self.data), len(attrs))
            text += "{} meta features".format(len(domain.metas)) \
                if len(domain.metas) else "(no meta features)"
        self.info_label.setText(text)

    def _check_data(self):
        self.clear_messages()
        if self.data and self.data.domain.has_discrete_attributes():
            self.data = None
            self.Error.discrete_attributes()
        if self.data and np.isnan(self.data.X).any():
            self.data.X = np.nan_to_num(self.data.X)
            self.Warning.missing_values()

    def _setup_model(self):
        estimator = ScBatchScorer()
        for var in self.data.domain.class_vars + self.data.domain.metas:
            if not var.is_primitive():
                continue
            try:
                score = float(estimator.score_data(self.data, var))
            except Exception:
                score = np.nan
            self.model.appendRow([
                self.__selected_item(var),
                self.__variable_item(var),
                self.__count_item(var),
                self.__score_item(score)
            ])

    def __selected_item(self, var):
        item = QStandardItem()
        item.setData(var, VariableRole)
        item.setCheckable(True)
        select = var in self.batch_vars
        item.setCheckState(Qt.Checked if select else Qt.Unchecked)
        item.setEditable(False)
        return item

    def __variable_item(self, var):
        item = QStandardItem()
        item.setData(var.name, Qt.DisplayRole)
        item.setData(gui.attributeIconDict[var], Qt.DecorationRole)
        item.setEditable(False)
        return item

    def __count_item(self, var):
        item = QStandardItem()
        if var.is_discrete:
            item.setData(len(var.values), Qt.DisplayRole)
        item.setEditable(False)
        return item

    def __score_item(self, score):
        item = QStandardItem()
        item.setData(score, Qt.DisplayRole)
        item.setEditable(False)
        return item

    def commit(self):
        data = None
        self.Error.general_error.clear()
        self.Warning.negative_values.clear()
        if self.data is not None:
            if (self.data.X < 0).any() and self.skip_zeros:
                self.Warning.negative_values()
                data = self.data
            else:
                try:
                    data = SCBatchNormalizer(
                        LinkMethod.items()[self.link_method],
                        self.skip_zeros, self.batch_vars
                    )(self.data)
                except Exception as e:
                    self.Error.general_error(str(e))
                    data = None
        self.Outputs.data.send(data)

    def send_report(self):
        method = LinkMethod.items()[self.link_method]
        if self.skip_zeros:
            method += " (Skip zero expressions)"
        variables = ", ".join([v.name for v in self.batch_vars]) \
            if self.batch_vars else "None"
        self.report_items("", [("Method", method),
                               ("Batch variable selection", variables)])


def main(args=None):
    app = QApplication(args or [])
    w = OWBatchNorm()
    w.set_data(Table("iris"))
    w.show()
    w.raise_()
    app.exec_()
    w.saveSettings()
    w.onDeleteWidget()


if __name__ == "__main__":
    sys.exit(main())



import unittest

from orangecontrib.single_cell.widgets.owscoregenes import OWRank
from Orange.widgets.tests.base import WidgetTest
from Orange.data import DiscreteVariable, ContinuousVariable, Domain, Table


class TestOWScoreGenes(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWRank)

    def test_data_no_attributes(self):
        """
        Do not crash when data has not attributes. Show an error.
        GH-51
        """
        w = self.widget

        class_var = DiscreteVariable('Stage name', values=['STG1', 'STG2'])
        metas = [ContinuousVariable('GeneName' + str(i)) for i in range(5)]
        domain = Domain(attributes=[], class_vars=class_var, metas=metas)

        data = Table.from_numpy(
            domain,
            [[0], [0], [0], [1]],
            metas=[[1, 2, 3, 4, 5], [4, 4, 4, 4, 4], [2, 3, 1, 1, 1], [-1, 0, 1, 0, 0]],
        )

        self.assertFalse(w.Error.no_attributes.is_shown())
        for is_shown, input_data in ((True, data), (False, None)):
            self.send_signal(w.Inputs.data, input_data)
            self.assertEqual(is_shown, w.Error.no_attributes.is_shown())


if __name__ == '__main__':
    unittest.main()



import numpy as np
import unittest
from itertools import product
from scipy.stats import pearsonr
from Orange.data import Table, ContinuousVariable, DiscreteVariable, Domain
from orangecontrib.single_cell.preprocess.scbnorm import *


class ScBatchNormalize(unittest.TestCase):

    def setUp(self):
        """ Set up datasets. Insert 0.5% random zeros. """
        np.random.seed(42)
        n = 100
        m = 200
        bias = 10
        self.noise = 0.1 * np.random.randn(n, 1)
        Z = np.hstack([np.random.randn(n, 1),
                       np.random.choice([0, 1], size=n).reshape(n, 1)])
        Y = np.random.randn(n, 1)
        W = np.random.randn(2, m)
        X_log = INV_LINKS[LINK_LOG](bias + Z.dot(W) + self.noise)
        X_lin = INV_LINKS[LINK_IDENTITY](bias + Z.dot(W) + self.noise)
        X_lin_neg = INV_LINKS[LINK_IDENTITY](Z.dot(W))

        self.zeros = (np.random.choice(range(n), size=100, replace=True),
                      np.random.choice(range(m), size=100, replace=True))
        X_log[self.zeros] = 0
        X_lin[self.zeros] = 0

        self.data_lin = Table.from_numpy(X=X_lin,
                                         metas=Z,
                                         Y=Y,
                                         domain=Domain(
                                            class_vars=[DiscreteVariable("class", values=["c1", "c2"])],
                                            attributes=[ContinuousVariable("X%d" % i) for i in range(m)],
                                            metas=[ContinuousVariable("Z0"),
                                                   DiscreteVariable("Z1", values=["a", "b"])]))

        self.data_log = Table.from_numpy(X=X_log,
                                         metas=Z,
                                         Y=Y,
                                         domain=Domain(
                                             class_vars=[DiscreteVariable("class", values=["c1", "c2"])],
                                             attributes=[ContinuousVariable("X%d" % i) for i in range(m)],
                                             metas=[ContinuousVariable("Z0"),
                                                    DiscreteVariable("Z1", values=["a", "b"])]))

        self.data_lin_neg = Table.from_numpy(X=X_lin_neg,
                                             metas=Z,
                                             Y=Y,
                                             domain=Domain(
                                                 class_vars=[DiscreteVariable("class", values=["c1", "c2"])],
                                                 attributes=[ContinuousVariable("X%d" % i) for i in range(m)],
                                                 metas=[ContinuousVariable("Z0"),
                                                        DiscreteVariable("Z1", values=["a", "b"])]))

    def test_multivariate_lin(self):
        """ Assert batch variables are decorrelated and the remaining factors (noise) become correlated. """
        alpha = 0.05
        model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"], link=LINK_IDENTITY)
        model.fit(self.data_lin)
        data1 = model(self.data_lin)
        assert pearsonr(data1.get_column_view("Z0")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(data1.get_column_view("Z1")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(self.noise.ravel(), data1.X[:, 0].ravel())[1] < alpha
        assert np.linalg.norm(data1.X[self.zeros]) == 0

    def test_multivariate_log(self):
        """ Assert batch variables are decorrelated and the remaining factors (noise) become correlated. """
        alpha = 0.05
        model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"], link=LINK_LOG)
        model.fit(self.data_log)
        data1 = model(self.data_log)
        assert pearsonr(data1.get_column_view("Z0")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(data1.get_column_view("Z1")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(self.noise.ravel(), data1.X[:, 0].ravel())[1] < alpha
        assert np.linalg.norm(data1.X[self.zeros]) == 0

    def test_preprocessor_lin(self):
        """ Test as a preprocessor (linear link). """
        pp = SCBatchNormalizer(batch_vars=["Z0", "Z1"], link=LINK_IDENTITY)
        data1 = pp(self.data_lin)
        data2 = self.data_lin.transform(data1.domain)
        np.testing.assert_almost_equal(data1.X, data2.X)
        assert np.linalg.norm(data1.X[self.zeros]) == 0

    def test_preprocessor_log(self):
        """ Test as a preprocessor (log link). """
        pp = SCBatchNormalizer(batch_vars=["Z0", "Z1"], link=LINK_LOG)
        data1 = pp(self.data_log)
        data2 = self.data_log.transform(data1.domain)
        np.testing.assert_almost_equal(data1.X, data2.X)
        assert np.linalg.norm(data1.X[self.zeros]) == 0

    def test_preprocessor_vs_model_log(self):
        """ Normalizer and model should return the same values (log). """
        pp = SCBatchNormalizer(batch_vars=["Z0", "Z1"], link=LINK_LOG)
        model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"], link=LINK_LOG)
        model.fit(self.data_log)
        data1 = model(self.data_log)
        data2 = pp(self.data_log)
        np.testing.assert_almost_equal(data1.X, data2.X)

    def test_preprocessor_vs_model_lin(self):
        """ Normalizer and model should return the same values (lin). """
        pp = SCBatchNormalizer(batch_vars=["Z0", "Z1"], link=LINK_IDENTITY, nonzero_only=False)
        model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"], link=LINK_IDENTITY, nonzero_only=False)
        model.fit(self.data_lin)
        data1 = model(self.data_lin)
        data2 = pp(self.data_lin)
        np.testing.assert_almost_equal(data1.X, data2.X)

    def test_nonzero_lin(self):
        """ Fit all including zeros (linear link). """
        alpha = 0.05
        model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"],
                                      nonzero_only=False,
                                      link=LINK_IDENTITY)
        model.fit(self.data_lin)
        data1 = model(self.data_lin)
        assert pearsonr(data1.get_column_view("Z0")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(data1.get_column_view("Z1")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(self.noise.ravel(), data1.X[:, 0].ravel())[1] < alpha
        assert np.linalg.norm(data1.X[self.zeros]) > 0

    def test_nonzero_log(self):
        """ Fit all including zeros (log link)"""
        def dummy_call():
            return ScBatchNormalizeModel(batch_vars=["Z0", "Z1"],
                                         nonzero_only=False,
                                         link=LINK_LOG)
        self.assertRaises(ValueError, dummy_call)

    def test_batch_class(self):
        """ Test including class variable. """
        alpha = 0.05
        pp = SCBatchNormalizer(batch_vars=("class", "Z0"))
        data1 = pp(self.data_lin)
        assert pearsonr(data1.get_column_view("class")[0], data1.X[:, 0].ravel())[1] > alpha
        assert pearsonr(data1.get_column_view("Z0")[0], data1.X[:, 0].ravel())[1] > alpha

    def test_no_vars(self):
        """ Test with no batch variables. """
        pp = SCBatchNormalizer(batch_vars=())
        data1 = pp(self.data_lin)
        assert np.linalg.norm(data1.X - self.data_lin.X) == 0

    def test_scorer_correlations(self):
        """ Matrix form correlations should equal default numpy implementations. """
        A = self.data_lin.X[:, :10]
        B = self.data_log.X[:, :10]
        C, P = ScBatchScorer.correlations(A, B)
        for i, j in product(range(A.shape[1]), range(B.shape[1])):
            c, p = pearsonr(A[:, i], B[:, j])
            assert abs(C[i, j] - c) < 1e-3
            assert abs(P[i, j] - p) < 1e-3

    def test_negative_lin(self):
        """ Assert batch variables are decorrelated and the remaining factors (noise) become correlated. """
        def dummy_call():
            model = ScBatchNormalizeModel(batch_vars=["Z0", "Z1"],
                                          nonzero_only=True,
                                          link=LINK_IDENTITY)
            model.fit(self.data_lin_neg)
        self.assertRaises(ValueError, dummy_call)


import unittest
from unittest.mock import patch

import pkg_resources
from packaging import version

from orangecontrib.single_cell.widgets.owscdatasets import OWscDataSets
from Orange.widgets.tests.base import WidgetTest


class TestOWscDataSets(WidgetTest):
    def test_widget_setup(self):
        orange_version = pkg_resources.get_distribution("orange3").version
        if version.parse(orange_version) > version.parse("3.27.0"):
            with patch("Orange.widgets.data.owdatasets.list_remote", lambda _: {}):
                self.widget = self.create_widget(OWscDataSets)
                self.wait_until_finished(self.widget)
        else:
            with patch.object(OWscDataSets, "list_remote", staticmethod(lambda: [])):
                self.widget = self.create_widget(OWscDataSets)
                self.wait_until_finished(self.widget)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

import numpy as np
import numpy.testing as npt
import pandas as pd

from Orange.data import DiscreteVariable, StringVariable

from orangecontrib.single_cell.widgets.load_data import (
    LoomLoader, ExcelLoader, MtxLoader, CountLoader, Loader, PickleLoader,
    CsvLoader, H5ADLoader, get_data_loader, Concatenate,
)


class TestLoadData(unittest.TestCase):
    def test_get_data_loader(self):
        self.assertIsInstance(get_data_loader("matrix.mtx"), MtxLoader)
        self.assertIsInstance(get_data_loader("lib.cell.count"), CountLoader)
        loader = get_data_loader("DATA_MATRIX_LOG_TPM.txt")
        self.assertIsInstance(loader, Loader)
        self.assertIsInstance(get_data_loader("data.xls"), ExcelLoader)
        self.assertIsInstance(get_data_loader("data.loom"), LoomLoader)
        self.assertIsInstance(get_data_loader("data_sparse.h5ad"), H5ADLoader)
        self.assertIsInstance(get_data_loader("data_dense.h5ad"), H5ADLoader)

    def test_get_data_loader_pickle(self):
        self.assertIsInstance(get_data_loader("data.pkl"), PickleLoader)
        self.assertIsInstance(get_data_loader("data.pickle"), PickleLoader)

    def test_get_data_loader_compressed_file(self):
        loader = get_data_loader("lib.cell.count.gz")
        self.assertIsInstance(loader, CountLoader)

    def test_file_summary_mtx(self):
        file_name = os.path.join(os.path.dirname(__file__),
                                 "data/10x/hg19/matrix.mtx")
        loader = MtxLoader(file_name)
        self.assertAlmostEqual(loader.file_size, 112, delta=15)
        self.assertEqual(loader.n_rows, 4)
        self.assertEqual(loader.n_cols, 6)
        self.assertEqual(loader.sparsity, 0.625)

    def test_file_summary_broad(self):
        file_name = os.path.join(os.path.dirname(__file__),
                                 "data/DATA_MATRIX_LOG_TPM.txt")
        loader = Loader(file_name)
        self.assertAlmostEqual(loader.file_size, 1084, delta=15)
        self.assertEqual(loader.n_rows, 10)
        self.assertEqual(loader.n_cols, 15)
        self.assertEqual(round(loader.sparsity, 2), 0.86)

    def test_file_summary_hhmi(self):
        file_name = os.path.join(os.path.dirname(__file__),
                                 "data/lib.cell.count")
        loader = CountLoader(file_name)
        self.assertAlmostEqual(loader.file_size, 428, delta=15)
        self.assertEqual(loader.n_rows, 10)
        self.assertEqual(loader.n_cols, 11)
        self.assertEqual(round(loader.sparsity, 2), 0.99)

    def test_file_summary_pickle(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data.pkl")
        loader = PickleLoader(file_name)
        self.assertEqual(loader.file_size, 5021)
        self.assertEqual(loader.n_rows, None)
        self.assertEqual(loader.n_cols, None)
        self.assertEqual(loader.sparsity, None)

    def test_file_summary_gz(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data.txt.gz")
        loader = Loader(file_name)
        self.assertEqual(loader.file_size, 361)
        self.assertEqual(loader.n_rows, 10)
        self.assertEqual(loader.n_cols, 15)
        self.assertEqual(round(loader.sparsity, 2), 0.86)

    def test_file_summary_xls(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data.xlsx")
        loader = ExcelLoader(file_name)
        self.assertEqual(loader.file_size, 9160)
        self.assertEqual(loader.n_rows, 11)
        self.assertEqual(loader.n_cols, 15)
        self.assertEqual(round(loader.sparsity, 2), 0.86)

    def test_file_summary_loom(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data.loom")
        loader = LoomLoader(file_name)
        self.assertEqual(loader.file_size, 14100)
        self.assertEqual(loader.n_rows, 10)
        self.assertEqual(loader.n_cols, 20)
        self.assertEqual(round(loader.sparsity, 2), 0.93)

    def test_file_summary_h5ad_sparse(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data_sparse.h5ad")
        loader = H5ADLoader(file_name)
        self.assertEqual(19560, loader.file_size)
        self.assertEqual(20, loader.n_rows)
        self.assertEqual(25, loader.n_cols)
        self.assertEqual(0.59, round(loader.sparsity, 2))

    def test_file_summary_h5ad_dense(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data_dense.h5ad")
        loader = H5ADLoader(file_name)
        self.assertEqual(11060, loader.file_size)
        self.assertEqual(20, loader.n_rows)
        self.assertEqual(25, loader.n_cols)
        self.assertEqual(0.59, round(loader.sparsity, 2))

    def test_load_data_mtx(self):
        file_name = os.path.join(os.path.dirname(__file__),
                                 "data/10x/mm10/matrix.mtx")
        loader = MtxLoader(file_name)
        df = pd.read_csv(file_name, sep=" ", header=None, skiprows=[0, 1])
        attrs, X, meta_df, meta_df_index = loader._load_data()
        array = np.zeros((df.iloc[0, 0], df.iloc[0, 1]))
        for i, series in df.iterrows():
            if i == 0:
                continue
            array[series.iloc[1] - 1, series.iloc[0] - 1] = series.iloc[2]
        npt.assert_array_equal(X, array)

    def test_load_data_xls(self):
        kwargs = {"header_rows": 0, "header_cols": 0}
        xls_path = os.path.join(os.path.dirname(__file__), "data/data.xlsx")
        xls_loader = ExcelLoader(xls_path)
        xls_attrs, xls_X, xls_M, xls_M_index = xls_loader._load_data(**kwargs)
        csv_loader = Loader(os.path.join(os.path.dirname(__file__),
                                         "data/DATA_MATRIX_LOG_TPM.txt"))
        csv_attrs, csv_X, csv_M, csv_M_index = csv_loader._load_data(**kwargs)
        self.assertEqual(xls_attrs, csv_attrs)
        npt.assert_array_almost_equal(xls_X, csv_X)
        npt.assert_array_equal(xls_M, csv_M)
        npt.assert_array_equal(xls_M_index, csv_M_index)

    def test_n_genes_n_cells(self):
        file_name = os.path.join(os.path.dirname(__file__),
                                 "data/10x/hg19/matrix.mtx")
        loader = get_data_loader(file_name)
        self.assertEqual(loader.n_genes, loader.n_rows)
        self.assertEqual(loader.n_cells, loader.n_cols)

        file_name = os.path.join(os.path.dirname(__file__),
                                 "DATA_MATRIX_LOG_TPM.txt")
        loader = get_data_loader(file_name)
        self.assertEqual(loader.n_genes, loader.n_rows)
        self.assertEqual(loader.n_cells, loader.n_cols)

    def test_copy_loader(self):
        loader = MtxLoader(os.path.join(
            os.path.dirname(__file__), "data/10x/hg19/matrix.mtx")
        )
        loader.sample_rows_enabled = True
        copied = loader.copy()
        self.assertIsInstance(copied, Loader)
        self.assertTrue(copied.sample_rows_enabled)

    def test_guess_metas_type(self):
        file_name = os.path.join(os.path.dirname(__file__), "data/data.csv")
        loader = CsvLoader(file_name)
        loader.header_rows_count = 1
        loader.header_cols_count = 3
        data = loader()
        self.assertEqual(data.domain.metas[0].name, "level_0")
        self.assertIsInstance(data.domain.metas[0], StringVariable)
        self.assertEqual(data.domain.metas[1].name, "barcode")
        self.assertIsInstance(data.domain.metas[1], StringVariable)
        self.assertEqual(data.domain.metas[2].name, "cluster")
        self.assertIsInstance(data.domain.metas[2], DiscreteVariable)
        arr = np.array(["cell_0001", "AAGTGAAAG-CGACTCCT", 1], dtype=object)
        npt.assert_array_equal(data.metas[0], arr)


class TestConcatenate(unittest.TestCase):
    def test_concatenate_intersection(self):
        data1 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/hg19/matrix.mtx"))()
        data2 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/mm10/matrix.mtx"))()
        data3 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/hg19/matrix.mtx"))()
        concat_data = Concatenate.concatenate(
            Concatenate.INTERSECTION,
            ((data1, "1"), (data2, "2"), (data3, "3"))
        )
        self.assertEqual(2 * len(data1) + len(data2), len(concat_data))
        self.assertEqual(len(concat_data.domain.attributes), 1)
        self.assertEqual(len(concat_data.domain.metas), 2)

    def test_concatenate_union(self):
        data1 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/hg19/matrix.mtx"))()
        data2 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/mm10/matrix.mtx"))()
        data3 = MtxLoader(os.path.join(os.path.dirname(__file__),
                                       "data/10x/hg19/matrix.mtx"))()
        concat_data = Concatenate.concatenate(
            Concatenate.UNION, ((data1, "1"), (data2, "2"), (data3, "3"))
        )
        self.assertEqual(2 * len(data1) + len(data2), len(concat_data))
        self.assertEqual(len(concat_data.domain.attributes), 8)
        self.assertEqual(len(concat_data.domain.metas), 2)



import numpy as np

from AnyQt.QtCore import Qt

from Orange.data import Table
from Orange.widgets.tests.base import WidgetTest
from Orange.widgets.tests.utils import simulate

from orangecontrib.single_cell.widgets.owbatchnorm import (
    OWBatchNorm, LinkMethod
)


class TestOWBatchNorm(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWBatchNorm)

    def test_valid_input(self):
        """Check batch normalization for data set with continuous attributes"""
        data = Table("iris")
        self.send_signal(self.widget.Inputs.data, data)
        self.assertEqual(self.widget.model.rowCount(), 1)
        self.widget.model.item(0).setCheckState(Qt.Checked)
        output = self.get_output(self.widget.Outputs.data)
        self.assertIsInstance(output, Table)
        self.assertFalse((output.X == data.X).any())

    def test_discrete_attributes_input(self):
        """Check batch normalization for data set with discrete attributes"""
        self.send_signal(self.widget.Inputs.data, Table("heart_disease"))
        self.assertIsNone(self.get_output(self.widget.Outputs.data))
        self.assertTrue(self.widget.Error.discrete_attributes.is_shown())
        self.send_signal(self.widget.Inputs.data, None)
        self.assertFalse(self.widget.Error.discrete_attributes.is_shown())
        self.assertIsNone(self.get_output(self.widget.Outputs.data))
        self.assertEqual(self.widget.info_label.text(), "No data on input.")

    def test_missing_values_input(self):
        """Check batch normalization for data set with unknown values """
        data = Table("iris")
        data[0, 3] = np.nan
        self.send_signal(self.widget.Inputs.data, data)
        self.assertIsInstance(self.get_output(self.widget.Outputs.data), Table)
        self.assertTrue(self.widget.Warning.missing_values.is_shown())
        self.send_signal(self.widget.Inputs.data, None)
        self.assertFalse(self.widget.Warning.missing_values.is_shown())
        self.assertIsNone(self.get_output(self.widget.Outputs.data))

    def test_negative_values_input_log_link(self):
        """Check batch normalization for data set with negative values"""
        data = Table("iris")
        data[0, 3] = -1
        self.send_signal(self.widget.Inputs.data, data)
        self.widget.model.item(0).setCheckState(Qt.Checked)
        link_method = self.widget.controls.link_method
        simulate.combobox_activate_index(link_method, LinkMethod.LOG_LINK)

        self.assertTrue(self.widget.Warning.negative_values.is_shown())
        output = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(output.X, data.X)

        self.send_signal(self.widget.Inputs.data, None)
        self.assertFalse(self.widget.Warning.negative_values.is_shown())

    def test_negative_values_input_id_link(self):
        """Check batch normalization for data set with negative values"""
        data = Table("iris")
        data[0, 3] = -1
        self.send_signal(self.widget.Inputs.data, data)
        self.widget.model.item(0).setCheckState(Qt.Checked)
        link_method = self.widget.controls.link_method
        simulate.combobox_activate_index(link_method, LinkMethod.IDENTITY_LINK)

        self.widget.skip_zeros_check.setChecked(True)
        self.assertTrue(self.widget.Warning.negative_values.is_shown())
        output = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(output.X, data.X)

        self.widget.skip_zeros_check.setChecked(False)
        self.assertFalse(self.widget.Warning.negative_values.is_shown())
        output = self.get_output(self.widget.Outputs.data)
        self.assertFalse((output.X == data.X).any())



import unittest
import numpy as np

from orangecontrib.single_cell.widgets.owscoregenes import OWRank
from Orange.widgets.tests.base import WidgetTest
from Orange.data import DiscreteVariable, ContinuousVariable, Domain, Table


class TestOWRank(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWRank)
        self.widget.selected_methods = set()

        class_var = DiscreteVariable('Stage name', values=['STG1', 'STG2'])
        attributes = [ContinuousVariable('GeneName' + str(i)) for i in range(2)]
        domain = Domain(attributes, class_vars=class_var)
        self.data = Table.from_numpy(
            domain, [[8, -4], [9, -2], [10, 0], [11, 2], [12, 4]], [[0], [0], [1], [1], [1]]
        )

        self.expected_mean = np.array([[10.0], [0.0]])
        self.expected_variance = np.array([[2.0], [8.0]])
        self.expected_dispersion = np.array([[0.2], [8.0]])

        # GeneName1(second column) -> if mean is zero, we set it to 1
        self.expected_cv = np.array([[0.141], [2.828]])

    def test_mean(self):
        self.widget.selected_methods.add('Mean')

        # input data
        self.send_signal(self.widget.Inputs.data, self.data)
        self.assertIsNotNone(self.widget.data)

        out_data = self.get_output(self.widget.Outputs.scores)
        self.assertTrue((out_data.X == self.expected_mean).all())

    def test_variance(self):
        self.widget.selected_methods.add('Variance')

        # input data
        self.send_signal(self.widget.Inputs.data, self.data)
        self.assertIsNotNone(self.widget.data)

        out_data = self.get_output(self.widget.Outputs.scores)
        self.assertTrue((out_data.X == self.expected_variance).all())

    def test_dispersion(self):
        self.widget.selected_methods.add('Dispersion')

        # input data
        self.send_signal(self.widget.Inputs.data, self.data)
        self.assertIsNotNone(self.widget.data)

        out_data = self.get_output(self.widget.Outputs.scores)
        self.assertTrue((out_data.X == self.expected_dispersion).all())

    def test_coef_of_variation(self):
        self.widget.selected_methods.add('Coef. of variation (CV)')

        # input data
        self.send_signal(self.widget.Inputs.data, self.data)
        self.assertIsNotNone(self.widget.data)

        out_data = self.get_output(self.widget.Outputs.scores)
        self.assertTrue((np.around(out_data.X, decimals=3) == self.expected_cv).all())


if __name__ == '__main__':
    unittest.main()



import os
import unittest

import numpy as np
import numpy.testing as npt
import pandas as pd

from Orange.data import ContinuousVariable, Variable
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.single_cell.widgets.owloaddata import OWLoadData


class TestOWLoadData(WidgetTest):
    def setUp(self):
        Variable._clear_all_caches()
        self.widget = self.create_widget(OWLoadData)
        self._path = os.path.join(os.path.dirname(__file__), "data")

    def test_widget_no_file(self):
        self.assertEqual(self.widget.summary_label.text(), "")

    def test_open_mtx(self):
        file_name = os.path.join(self._path, "10x/mm10/matrix.mtx")
        self.widget.set_current_path(file_name)
        text = self.widget.summary_label.text()
        self.assertIn("5 rows, 5 columns", text)
        self._check_headers_and_row_labels_box((0, False, 0, False))
        self._check_input_data_structure_box((True, False, False, False))
        self._check_sample_data_box()
        values = ((True, False, "barcodes.tsv", False),
                  (True, False, "genes.tsv", False))
        self._check_annotation_files_box(values)

    def test_open_broad(self):
        file_name = os.path.join(self._path, "DATA_MATRIX_LOG_TPM.txt")
        self.widget.set_current_path(file_name)
        text = self.widget.summary_label.text()
        self.assertIn("10 rows, 15 columns", text)
        self._check_headers_and_row_labels_box((1, True, 1, True))
        self._check_input_data_structure_box((True, True, False, True))
        self._check_sample_data_box()
        values = ((False, True, "", False),
                  (False, True, "", False))
        self._check_annotation_files_box(values)

    def test_open_xls(self):
        file_name = os.path.join(self._path, "data.xlsx")
        self.widget.set_current_path(file_name)
        text = self.widget.summary_label.text()
        self.assertIn("11 rows, 15 columns", text)
        self._check_headers_and_row_labels_box((1, True, 1, True))
        self._check_input_data_structure_box((True, True, False, True))
        self._check_sample_data_box()
        values = ((False, True, "", False),
                  (False, True, "", False))
        self._check_annotation_files_box(values)

    def test_open_hhmi(self):
        file_name = os.path.join(self._path, "lib.cell.count")
        self.widget.set_current_path(file_name)
        text = self.widget.summary_label.text()
        self.assertIn("10 rows, 11 columns", text)
        self._check_headers_and_row_labels_box((1, False, 1, False))
        self._check_input_data_structure_box((True, False, False, False))
        self._check_sample_data_box()
        values = ((True, True, "lib.cell.meta", True),
                  (False, True, "", False))
        self._check_annotation_files_box(values)

    def test_load_data_mtx(self):
        file_name_mtx = os.path.join(self._path, "10x/hg19/matrix.mtx")
        self.widget.set_current_path(file_name_mtx)
        self.widget.commit()
        data = self.get_output("Data")
        file_name = os.path.join(self._path, "10x/hg19/genes.tsv")
        df = pd.read_csv(file_name, sep="\t", header=None)
        self._test_load_data_mtx_attributes(data.domain.attributes, df)
        df = pd.read_csv(file_name_mtx, sep=" ", header=None, skiprows=[0, 1])
        self._test_load_data_mtx_x(data.X, df)
        file_name = os.path.join(self._path, "10x/hg19/barcodes.tsv")
        df = pd.read_csv(file_name, sep="\t", header=None)
        self._test_load_data_metas(data.metas, df)

    def test_load_data_mtx_sample(self):
        file_name_mtx = os.path.join(self._path, "10x/mm10/matrix.mtx")
        self.widget.set_current_path(file_name_mtx)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(10)
        self.widget.set_sample_cols_p(10)
        self.widget.commit()
        data = self.get_output("Data")
        file_name = os.path.join(self._path, "10x/mm10/genes.tsv")
        df = pd.read_csv(file_name, sep="\t", header=None, skiprows=[4])
        self._test_load_data_mtx_attributes(data.domain.attributes, df)
        df = pd.read_csv(file_name_mtx, sep=" ", header=None,
                         skiprows=[0, 1, 9, 10])
        df.iloc[0, 0] = 4
        df.iloc[0, 1] = 4
        df.iloc[0, 2] = 6
        self._test_load_data_mtx_x(data.X, df)
        file_name = os.path.join(self._path, "10x/mm10/barcodes.tsv")
        df = pd.read_csv(file_name, sep="\t", header=None, skiprows=[4])
        self._test_load_data_metas(data.metas, df)

    def test_load_data_broad(self):
        file_name = os.path.join(self._path, "DATA_MATRIX_LOG_TPM.txt")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=0)
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_xls(self):
        file_name = os.path.join(self._path, "data.xlsx")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_excel(file_name, header=0, index_col=0)
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_compressed(self):
        file_name = os.path.join(self._path, "data.txt.gz")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=0)
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_pickle(self):
        file_name = os.path.join(self._path, "data.pkl")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        data = self.get_output("Data")
        file_name = os.path.join(self._path, "data.txt.gz")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=0)
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_loom(self):
        file_name = os.path.join(self._path, "data.loom")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        self.assertEqual(self.get_output("Data").X.shape, (20, 10))
        self.assertEqual(self.get_output("Data").metas.shape, (20, 1))

    def test_load_data_broad_sample(self):
        file_name = os.path.join(self._path, "DATA_MATRIX_LOG_TPM.txt")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(40)
        self.widget.set_sample_cols_p(60)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_csv(
            file_name, header=0, sep="\t", index_col=0,
            usecols=[0, 1, 2, 3, 5, 7, 12, 14], skiprows=[4, 5, 6, 7, 9, 10])
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_xls_sample(self):
        file_name = os.path.join(self._path, "data.xlsx")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(40)
        self.widget.set_sample_cols_p(60)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_excel(
            file_name, header=0, index_col=0,
            usecols=[0, 1, 2, 3, 5, 7, 12, 14], skiprows=[4, 5, 6, 7, 9, 10])
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        self._test_load_data_broad_metas(data.metas, df)

    def test_load_data_hhmi(self):
        file_name = os.path.join(self._path, "lib.cell.count")
        self.widget.set_current_path(file_name)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=0)
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        file_name = os.path.join(self._path, "lib.cell.meta")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=None)
        df.iloc[:, [1, 3]] = 0
        df.iloc[:, 6] = [0, 2, 0, 2, 0, 2, 0, 1, 0, 1]
        self._test_load_data_metas(data.metas, df)

    def test_load_data_hhmi_sample(self):
        file_name = os.path.join(self._path, "lib.cell.count")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(40)
        self.widget.set_sample_cols_p(60)
        self.widget.commit()
        data = self.get_output("Data")
        df = pd.read_csv(
            file_name, header=0, sep="\t", index_col=0,
            usecols=[0, 1, 2, 3, 5, 6], skiprows=[6, 8, 9, 10])
        self._test_load_data_attributes(data.domain.attributes, df)
        self._test_load_data_x(data.X, df)
        file_name = os.path.join(self._path, "lib.cell.meta")
        df = pd.read_csv(file_name, header=0, sep="\t", index_col=None,
                         skiprows=[4, 6, 8, 9, 10])
        df.iloc[:, [1, 3]] = 0
        df.iloc[:, 5] = [1, 0, 1, 1, 1]
        df.iloc[:, 6] = [0, 1, 0, 0, 0]
        self._test_load_data_metas(data.metas, df)

    def test_load_data_loom_sample(self):
        file_name = os.path.join(self._path, "data.loom")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(40)
        self.widget.set_sample_cols_p(60)
        self.widget.commit()
        data = self.get_output("Data")
        X = np.array([[4, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0],
                      [0, 0, 0, 0, 0, 0], [0, 7, 0, 0, 0, 0],
                      [0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0],
                      [0, 0, 0, 0, 5, 0], [9, 0, 0, 0, 0, 0],
                      [0, 0, 0, 0, 6, 0], [0, 0, 0, 0, 0, 0],
                      [0, 0, 0, 0, 0, 0]])
        npt.assert_array_equal(data.X, X)
        metas = np.array([[0, 1, 2, 3, 4, 5, 6, 7, 8, 13, 16]]).T
        npt.assert_array_equal(data.metas, metas)
        self.assertListEqual([attr.name for attr in data.domain.attributes],
                             ["0", "1", "2", "3", "6", "7"])

    def test_load_data_h5ad_sparse_sample(self):
        file_name = os.path.join(self._path, "data_sparse.h5ad")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(30)
        self.widget.set_sample_cols_p(10)
        self.widget.commit()
        data = self.get_output("Data")
        X = np.array([[0, 2, 1, 0], [1, 0, 0, 0], [2, 1, 2, 1], [1, 1, 0, 0],
                      [1, 0, 0, 0], [0, 1, 0, 2], [1, 0, 0, 1], [0, 2, 0, 1],
                      [0, 1, 1, 0]])
        npt.assert_array_equal(data.X, X)
        metas = np.array([[0, 0, 10], [1, 0, 10], [2, 0, 15], [3, 0, 10],
                          [6, 1, 11], [7, 1, 15], [8, 1, 11], [9, 1, 20],
                          [14, 2, 12]])
        npt.assert_array_equal(data.metas, metas)
        self.assertListEqual([attr.name for attr in data.domain.attributes],
                             ["Gene 0", "Gene 1", "Gene 2", "Gene 3"])

    def test_load_data_h5ad_dense_sample(self):
        file_name = os.path.join(self._path, "data_dense.h5ad")
        self.widget.set_current_path(file_name)
        self.widget.sample_rows_cb.setChecked(True)
        self.widget.sample_cols_cb.setChecked(True)
        self.widget.set_sample_rows_p(30)
        self.widget.set_sample_cols_p(10)
        self.widget.commit()
        data = self.get_output("Data")
        X = np.array([[0, 2, 1, 0], [1, 0, 0, 0], [2, 1, 2, 1], [1, 1, 0, 0],
                      [1, 0, 0, 0], [0, 1, 0, 2], [1, 0, 0, 1], [0, 2, 0, 1],
                      [0, 1, 1, 0]])
        npt.assert_array_equal(data.X, X)
        metas = np.array([[0, 0, 10], [1, 0, 10], [2, 0, 15], [3, 0, 10],
                          [6, 1, 11], [7, 1, 15], [8, 1, 11], [9, 1, 20],
                          [14, 2, 12]])
        npt.assert_array_equal(data.metas, metas)
        self.assertListEqual([attr.name for attr in data.domain.attributes],
                             ["Gene 0", "Gene 1", "Gene 2", "Gene 3"])

    def test_not_enough_headers(self):
        file_name = os.path.join(self._path, "DATA_MATRIX_LOG_TPM.txt")
        self.widget.set_current_path(file_name)
        self.widget.set_header_rows_count(0)
        self.widget.commit()
        self.assertTrue(self.widget.Error.inadequate_headers.is_shown())
        self.widget.set_header_cols_count(0)
        self.widget.commit()
        self.assertTrue(self.widget.Error.inadequate_headers.is_shown())
        self.widget.set_header_rows_count(1)
        self.widget.set_header_cols_count(1)
        self.widget.commit()
        self.assertFalse(self.widget.Error.inadequate_headers.is_shown())

    def _check_headers_and_row_labels_box(self, values):
        self.assertEqual(self.widget.header_rows_spin.value(), values[0])
        self.assertEqual(self.widget.header_rows_spin.isEnabled(), values[1])
        self.assertEqual(self.widget.header_cols_spin.value(), values[2])
        self.assertEqual(self.widget.header_cols_spin.isEnabled(), values[3])

    def _check_input_data_structure_box(self, values):
        rbs = self.widget.data_struct_box.children()[1].buttons
        self.assertEqual(rbs[0].isChecked(), values[0])
        self.assertEqual(rbs[0].isEnabled(), values[1])
        self.assertEqual(rbs[1].isChecked(), values[2])
        self.assertEqual(rbs[1].isEnabled(), values[3])

    def _check_sample_data_box(self):
        self.assertFalse(self.widget.sample_rows_cb.isChecked())
        self.assertTrue(self.widget.sample_rows_cb.isEnabled())
        self.assertFalse(self.widget.sample_cols_cb.isChecked())
        self.assertTrue(self.widget.sample_cols_cb.isEnabled())

    def _check_annotation_files_box(self, v):
        self.assertEqual(self.widget.row_annotations_cb.isChecked(), v[0][0])
        self.assertEqual(self.widget.row_annotations_cb.isEnabled(), v[0][1])
        text = self.widget.row_annotations_combo.currentText()
        self.assertEqual(text, v[0][2])
        self.assertEqual(self.widget.row_annotations_combo.isEnabled(), v[0][3])
        self.assertEqual(self.widget.col_annotations_cb.isChecked(), v[1][0])
        self.assertEqual(self.widget.col_annotations_cb.isEnabled(), v[1][1])
        text = self.widget.col_annotations_combo.currentText()
        self.assertEqual(text, v[1][2])
        self.assertEqual(self.widget.col_annotations_combo.isEnabled(), v[1][3])

    def _test_load_data_attributes(self, attributes, df):
        for i, attribute in enumerate(attributes):
            self.assertIsInstance(attribute, ContinuousVariable)
            self.assertEqual(attribute.name, df.index[i])
            self.assertDictEqual(attribute.attributes, {})

    def _test_load_data_x(self, x, df):
        npt.assert_array_almost_equal(x, df.values.T)

    def _test_load_data_metas(self, metas, df):
        npt.assert_array_equal(metas, df.values)

    def _test_load_data_mtx_attributes(self, attributes, df):
        for i, attribute in enumerate(attributes):
            self.assertIsInstance(attribute, ContinuousVariable)
            self.assertEqual(attribute.name, df.iloc[i, 0])
            self.assertEqual(attribute.attributes["Id"], df.iloc[i, 0])
            self.assertEqual(attribute.attributes["Gene"], df.iloc[i, 1])

    def _test_load_data_mtx_x(self, x, df):
        array = np.zeros((df.iloc[0, 1], df.iloc[0, 0]))
        for i, series in df.iterrows():
            if i == 0:
                continue
            array[series.iloc[1] - 1, series.iloc[0] - 1] = series.iloc[2]
        npt.assert_array_equal(x, array)

    def _test_load_data_broad_metas(self, metas, data_frame):
        npt.assert_array_equal(metas.flatten(), data_frame.columns.values)

    # def test_valid_output(self):
    #     widget = self.create_widget(OWDataTable)
    #     self.widget.set_current_path(os.path.join(self._path, "data.xlsx"))
    #     self.widget.set_header_rows_count(3)
    #     self.widget.set_header_cols_count(2)
    #     self.widget.sample_rows_cb.setChecked(True)
    #     self.widget.sample_cols_cb.setChecked(True)
    #     self.widget.set_sample_rows_p(40)
    #     self.widget.set_sample_cols_p(60)
    #     self.widget.commit()
    #     data = self.get_output("Data")
    #     self.assertIsNotNone(data)
    #     self.assertEqual(data.X.shape, (3, 7))
    #     widget.set_dataset(data)


if __name__ == "__main__":
    unittest.main()


import unittest
import numpy as np
from orangecontrib.single_cell.preprocess.alignment import SeuratAlignmentModel, GENE_SCORING_METHODS


class TestAlignment(unittest.TestCase):

    def setUp(self):
        m = 50
        n = 70
        p = 40
        self.X = np.random.randint(0, 10, size=(m + n) * p).reshape(((m + n), p))
        self.y = np.zeros((m+n, ))
        self.y[m:] = 1


    def test_seurat_alignment(self):
        """ Test on sample data with included scoring methods. """
        for method in GENE_SCORING_METHODS:
            model = SeuratAlignmentModel(n_components=10,
                                         n_metagenes=20,
                                         gene_scoring=method,
                                         random_state=42)
            Z = model.fit_transform(self.X, self.y)
            assert Z.shape == (self.X.shape[0], model.n_components)
            assert np.isnan(Z).sum() == 0


if __name__ == "__main__":
    unittest.main()


import warnings
import os
import unittest
import numpy as np
import numpy.testing as npt

from Orange.data import Table, Domain
from Orange.widgets.tests.utils import table_dense_sparse
from orangecontrib.single_cell.preprocess.scpreprocess import (
    LogarithmicScale,
    Binarize,
    NormalizeSamples,
    NormalizeGroups,
    Standardize,
    SelectMostVariableGenes,
    DropoutGeneSelection,
    DropoutWarning,
)


class TestLogarithmicScale(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.table = Table.from_numpy(None, np.arange(1, 13).reshape((3, 4)))

    @table_dense_sparse
    def test_default(self, prepare_table):
        table = prepare_table(self.table)
        new_table = LogarithmicScale()(table)
        self.assertIsInstance(new_table, Table)
        npt.assert_array_equal(new_table, np.log2(1 + self.table.X))
        self.assertNotEqual(new_table, self.table)

    @table_dense_sparse
    def test_options(self, prepare_table):
        table = prepare_table(self.table)
        new_table = LogarithmicScale(LogarithmicScale.NaturalLog)(table)
        npt.assert_allclose(new_table, np.log(1 + self.table.X))

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        new_table = LogarithmicScale(LogarithmicScale.NaturalLog)(table)
        self.assertEqual(is_sparse, new_table.is_sparse())


class TestBinarize(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.table = Table.from_numpy(None, np.arange(12).reshape((3, 4)))

    @table_dense_sparse
    def test_default(self, prepare_table):
        table = prepare_table(self.table)
        new_table = Binarize()(table)
        self.assertIsInstance(new_table, Table)
        self.assertEqual(new_table.X[0, 0], 0)
        self.assertEqual(np.sum(new_table[1]), 4)
        self.assertNotEqual(new_table, self.table)

    @table_dense_sparse
    def test_options(self, prepare_table):
        table = prepare_table(self.table)
        new_table = Binarize(Binarize.Greater, 5)(table)
        self.assertEqual(np.sum(new_table.X[0]), 0)
        self.assertEqual(np.sum(new_table.X[1]), 2)
        self.assertEqual(np.sum(new_table.X[2]), 4)

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        new_table = Binarize(Binarize.Greater, 5)(table)
        self.assertEqual(is_sparse, new_table.is_sparse())


class TestNormalizeSamples(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.table = Table.from_numpy(
            None,
            np.array(
                [
                    [1.5, 8.5, 0],
                    [2, 0, 3],
                    [3, np.nan, 7],
                ]
            ),
        )

    @table_dense_sparse
    def test_default(self, prepare_table):
        table = prepare_table(self.table)
        new_table = NormalizeSamples()(table)
        self.assertIsInstance(new_table, Table)
        self.assertNotEqual(new_table, self.table)

        new_table = new_table.to_dense()
        npt.assert_array_almost_equal(new_table, [[150000, 850000, 0], [400000, 0, 600000], [300000, np.nan, 700000]])

    @table_dense_sparse
    def test_options(self, prepare_table):
        table = prepare_table(self.table)
        new_table = NormalizeSamples(NormalizeSamples.Median)(table)
        npt.assert_array_almost_equal(
            new_table,
            [[1.5, 8.5, 0], [4, 0, 6], [3, np.nan, 7]],
        )

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        new_table = NormalizeSamples(NormalizeSamples.Median)(table)
        self.assertEqual(is_sparse, new_table.is_sparse())


class TestNormalizeGroups(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        table = Table.from_numpy(None, [[1, 4, 0], [2, 5, 0], [3, 6, 1]])
        domain = Domain(table.domain.attributes[:-1], metas=table.domain.attributes[-1:])
        cls.table = table.transform(domain)

    @table_dense_sparse
    def test_default(self, prepare_table):
        table = prepare_table(self.table)
        new_table = NormalizeGroups(-1)(table)
        self.assertIsInstance(new_table, Table)
        self.assertNotEqual(new_table, self.table)

        new_table = new_table.to_dense()
        npt.assert_array_almost_equal(
            new_table.X, [[83333.333, 333333.333], [166666.667, 416666.667], [333333.333, 666666.667]], decimal=3
        )

    @table_dense_sparse
    def test_options(self, prepare_table):
        table = prepare_table(self.table)
        new_table = NormalizeGroups(-1, NormalizeGroups.Median)(table)
        new_table = new_table.to_dense()
        npt.assert_array_equal(new_table.X, [[0.5, 2], [1, 2.5], [2, 4]])

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        new_table = NormalizeGroups(-1, NormalizeGroups.Median)(table)
        self.assertEqual(is_sparse, new_table.is_sparse())


class TestStandardize(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.table = Table.from_numpy(None, np.arange(1, 7).reshape((2, 3)).T)

    def test_default(self):
        table = Standardize()(self.table)
        self.assertIsInstance(table, Table)
        self.assertNotEqual(table, self.table)
        npt.assert_array_almost_equal(table, [[-1.225, -1.225], [0.0, 0.0], [1.225, 1.225]], decimal=3)

    def test_options(self):
        table = Standardize(-1, 1)(self.table)
        npt.assert_array_equal(table, [[-1, -1], [0.0, 0.0], [1, 1]])


class TestSelectMostVariableGenes(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        path = os.path.join(os.path.dirname(__file__), '..', "data")
        data = Table(os.path.join(path, 'dermatology.tab'))
        cls.table = data.transform(Domain(data.domain.attributes[:-1], data.domain.class_vars))
        cls.variance = np.nanvar(cls.table.X, axis=0)
        cls.mean = np.nanmean(cls.table.X, axis=0)

    @table_dense_sparse
    def test_default(self, prepare_table):
        table = Table.from_numpy(None, np.arange(12).reshape((3, 4)).T ** (1 / 2))
        table = prepare_table(table)
        pp_table = SelectMostVariableGenes(n_genes=2, n_groups=2)(table)
        self.assertIsInstance(pp_table, Table)
        self.assertNotEqual(pp_table, table)
        npt.assert_array_equal(pp_table, table[:, :2])

    @table_dense_sparse
    def test_options_dispersion(self, prepare_table):
        attrs, cls = self.table.domain.attributes, self.table.domain.class_vars
        inds = sorted(np.argsort(self.variance / self.mean))
        data = self.table.transform(Domain(tuple(np.array(attrs)[inds]), cls))

        table = prepare_table(data)
        filtered_data = SelectMostVariableGenes(method=SelectMostVariableGenes.Dispersion, n_groups=None)(table)
        npt.assert_array_equal(filtered_data, data)

    @table_dense_sparse
    def test_options_variance(self, prepare_table):
        attrs, cls = self.table.domain.attributes, self.table.domain.class_vars
        inds = sorted(np.argsort(self.variance))
        table = self.table.transform(Domain(tuple(np.array(attrs)[inds]), cls))
        table = prepare_table(table)

        filtered_table = SelectMostVariableGenes(method=SelectMostVariableGenes.Dispersion, n_groups=None)(table)

        npt.assert_array_equal(filtered_table, table)

    @table_dense_sparse
    def test_options_mean(self, prepare_table):
        attrs, cls = self.table.domain.attributes, self.table.domain.class_vars
        inds = sorted(np.argsort(self.mean))
        table = self.table.transform(Domain(tuple(np.array(attrs)[inds]), cls))
        table = prepare_table(table)

        filtered_table = SelectMostVariableGenes(method=SelectMostVariableGenes.Dispersion, n_groups=None)(table)

        npt.assert_array_equal(filtered_table, table)

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        filtered_table = SelectMostVariableGenes(method=SelectMostVariableGenes.Dispersion, n_groups=None)(table)
        self.assertEqual(is_sparse, filtered_table.is_sparse())


class TestDropoutGeneSelection(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        path = os.path.join(os.path.dirname(__file__), '..', "data")
        cls.table = Table(os.path.join(path, 'dermatology.tab'))

    @table_dense_sparse
    def test_default(self, prepare_table):
        n_genes = 2
        table = prepare_table(self.table)
        filtered_table = DropoutGeneSelection(n_genes)(table)
        self.assertIsInstance(filtered_table, Table)
        self.assertEqual(len(filtered_table.domain.attributes), n_genes)

    @table_dense_sparse
    def test_detection(self, prepare_table):
        table = prepare_table(self.table)
        preprocessor = DropoutGeneSelection(n_genes=2)
        zero_rate, mean_expr = preprocessor.detection(table.X)
        self.assertEqual(len(zero_rate), len(table.domain.attributes))
        self.assertEqual(len(mean_expr), len(table.domain.attributes))
        npt.assert_allclose(
            zero_rate[:5],
            np.array([0.01092896, 0.02185792, 0.16120219, 0.32240437, 0.61202186]),
            atol=1e-7,
        )
        npt.assert_allclose(
            mean_expr[:5],
            np.array([0.9879741, 0.77491075, 0.78471751, 0.88894012, 0.58119243]),
            atol=1e-7,
        )

    @table_dense_sparse
    def test_warning(self, prepare_table):
        n_genes = 30
        table = prepare_table(self.table)
        with warnings.catch_warnings(record=True) as w:
            filtered_table = DropoutGeneSelection(n_genes)(table)
        self.assertIsInstance(filtered_table, Table)
        self.assertNotEqual(len(filtered_table.domain.attributes), n_genes)
        dropout_warnings = [warning for warning in w if issubclass(warning.category, DropoutWarning)]
        self.assertEqual(len(dropout_warnings), 1)

    @table_dense_sparse
    def test_preserves_density(self, prepare_table):
        table = prepare_table(self.table)
        is_sparse = table.is_sparse()
        filtered_table = DropoutGeneSelection(3)(table)
        self.assertEqual(is_sparse, filtered_table.is_sparse())



from sklearn import datasets
from Orange.data import ContinuousVariable, DiscreteVariable
import numpy as np

from Orange.data import Table, Domain
from Orange.widgets.tests.base import WidgetTest
from Orange.widgets.tests.utils import simulate

from orangecontrib.single_cell.widgets.owaligndatasets import OWAlignDatasets


class TestOWMDA(WidgetTest):

    def generate_dataset(self, n=3, cols=10, samples=1000):
        cluster_std = 2.5
        f = lambda x, i: .5 * x * i + 2 * i

        X, Y = datasets.make_blobs(n_samples=samples, n_features=cols, centers=1,
                                   cluster_std=cluster_std, random_state=10)
        for i in range(samples):
            y = i % n
            Y[i] = y
            if y > 0:
                X[i, cols // 2:] = f(X[i, cols // 2:], i)
        dom = Domain([ContinuousVariable.make(str(i)) for i in range(cols)],
                     DiscreteVariable.make('class', values=[str(i) for i in range(n)]))
        return Table.from_numpy(dom, X, Y[np.newaxis].T)

    def setUp(self):
        self.widget = self.create_widget(OWAlignDatasets)

    def test_for_nans(self):
        """ Check if output transformed data contains any NaNs. """
        data = self.generate_dataset()
        self.send_signal(self.widget.Inputs.data, data)

        gc = self.get_output(self.widget.Outputs.genes_components)
        td = self.get_output(self.widget.Outputs.transformed_data)

        self.assertIsNotNone(gc)
        self.assertIsNotNone(td)

        self.assertFalse(np.isnan(gc.X).any())

    def test_shape(self):
        """ Check shape of outputs. """
        n = 3
        cols = 10
        samples = 1000
        data = self.generate_dataset(n=n, cols=cols, samples=samples)
        self.send_signal(self.widget.Inputs.data, data)

        ngenes = cols
        ncomponents = cols // 2
        self.widget.ngenes = ngenes
        self.widget.ncomponents = ncomponents
        self.widget.fit()

        gc = self.get_output(self.widget.Outputs.genes_components)
        td = self.get_output(self.widget.Outputs.transformed_data)

        self.assertEqual(td.X.shape, (samples, ncomponents))
        self.assertEqual(gc.X.shape, (ngenes, ncomponents))

    def test_source_id_defult(self):
        """
        Check whether Source ID is set as a default source ID attribute.
        """
        w = self.widget
        data = self.generate_dataset()
        data1 = Table("iris")
        data = Table(
            Domain(
                data.domain.attributes, data.domain.class_var,
                metas=[DiscreteVariable("Source ID", values=["a", "b"])]),
            data.X, data.Y, metas=np.random.randint(0, 2, (len(data), 1)))
        data1 = Table(
            Domain(
                data1.domain.attributes, data1.domain.class_var,
                metas=[DiscreteVariable("Source ID", values=["a", "b", ])]),
            data1.X, data1.Y, metas=np.random.randint(0, 2, (len(data1), 1)))
        w._reset_settings()
        self.send_signal(w.Inputs.data, data)
        # without manual fix source ID would be class since it is first in list
        self.assertEqual("Source ID", str(w.source_id))
        # we change setting manually (it is stored in widget settings) and it should stay even when new signal comes
        cbox = w.controls.source_id
        simulate.combobox_activate_item(cbox, "class")
        self.assertEqual("class", str(w.source_id))
        self.send_signal(w.Inputs.data, None)
        self.send_signal(w.Inputs.data, data)
        self.assertEqual("class", str(w.source_id))

        self.send_signal(w.Inputs.data, data1)
        self.assertEqual("Source ID", str(w.source_id))

        self.send_signal(w.Inputs.data, data)
        self.assertEqual("class", str(w.source_id))



import os
import numbers
from collections import namedtuple
from typing import Dict, Tuple, List, Optional
from serverfiles import sizeformat

from AnyQt.QtCore import (
    Qt, QItemSelectionModel, pyqtSlot as Slot, QModelIndex, Signal
)
from AnyQt.QtGui import (
    QStandardItemModel, QStandardItem, QIcon, QBrush, QColor
)
from AnyQt.QtWidgets import (
    QStyledItemDelegate, QTreeView, QSizePolicy,
    QStyle, QHeaderView, QLineEdit, QFileDialog
)
from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import Msg

from orangecontrib.single_cell.widgets.load_data import Concatenate, Loader
import orangecontrib.single_cell.widgets.owloaddata as owloaddata

LoaderObjectRole = next(gui.OrangeUserRole)


class FileDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, str):
            option.text = os.path.split(data)[1]


class SizeDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, numbers.Integral):
            option.text = sizeformat(int(data))
            option.displayAlignment = Qt.AlignRight | Qt.AlignVCenter


class NumericalDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, numbers.Number):
            option.displayAlignment = Qt.AlignRight | Qt.AlignVCenter


class SparsityDelegate(QStyledItemDelegate):
    def initStyleOption(self, option, index):
        super().initStyleOption(option, index)
        data = index.data(Qt.DisplayRole)
        if isinstance(data, numbers.Number):
            option.text = "~" + str(round(data * 100)) + " %"
            option.displayAlignment = Qt.AlignRight | Qt.AlignVCenter


class RecentLineEdit(QLineEdit):
    def __init__(self, parent):
        super().__init__(parent)
        self.setText("Recent files")
        self.setReadOnly(True)

    def mouseReleaseEvent(self, event):
        self.parent().showPopup()


class MultiSampleModel(QStandardItemModel):
    def supportedDropActions(self):
        return Qt.MoveAction

    def supportedDragActions(self):
        return Qt.MoveAction

    def dropMimeData(self, data, action, row, column, parent):
        column = 0
        if parent.row() != -1:
            row = parent.row()
        return super().dropMimeData(data, action, row, column, QModelIndex())


class MultiSampleTreeView(QTreeView):
    drop_finished = Signal()
    data_changed = Signal()

    def __init__(self, parent):
        self._parent = parent
        super().__init__(
            sortingEnabled=True,
            selectionMode=QTreeView.SingleSelection,
            selectionBehavior=QTreeView.SelectRows,
            alternatingRowColors=True
        )
        self.setRootIsDecorated(False)
        self.setDragEnabled(True)
        self.setDragDropMode(QTreeView.InternalMove)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

        else:
            super().dragEnterEvent(event)

    def dropEvent(self, event):
        if event.mimeData().hasUrls():
            for url in event.mimeData().urls():
                event.acceptProposedAction()
                if os.path.isfile(url.toLocalFile()):
                    self._parent.set_current_path(url.toLocalFile())
            self._parent.select_last_item()
            self.drop_finished.emit()
        else:
            super().dropEvent(event)

    def startDrag(self, action):
        super().startDrag(action)
        self.drop_finished.emit()

    def commitData(self, event):
        super().commitData(event)
        self.data_changed.emit()


class OWMultiSample(owloaddata.OWLoadData):
    name = "Load Data"
    description = "Load samples for multi-sample analysis"
    icon = "icons/LoadData.svg"
    priority = 110

    class Information(owloaddata.OWLoadData.Information):
        file_in_list = Msg("File {} already in the list.")

    HEADER_SCHEMA = (
        ("selected", ""),
        ("name", "File"),
        ("source", "Source Name"),
        ("cells", "Cells"),
        ("genes", "Genes"),
        ("size", "Size"),
        ("sparsity", "Sparsity"),
        ("remove", "")
    )
    OUTPUT_TYPES = ["Intersection", "Union"]

    # override settings
    _recent_row_annotations = []
    _recent_col_annotations = []
    _cells_in_rows = False
    _col_annotations_enabled = False
    _row_annotations_enabled = False
    _last_path = ""
    _header_rows_count = 1
    _header_cols_count = 1
    _sample_rows_enabled = False
    _sample_cols_enabled = False
    _sample_cols_p = 10.0
    _sample_rows_p = 10.0

    samples = Setting([])  # type: List[Tuple[str, str, bool]]
    loaders = Setting({})  # type: Dict[str, Loader]
    output_type = Setting(Concatenate.INTERSECTION)
    auto_commit = Setting(False)

    resizing_enabled = True

    def __init__(self):

        self._header_labels = hls = [label for _, label in self.HEADER_SCHEMA]
        header = namedtuple("header", [tag for tag, _ in self.HEADER_SCHEMA])
        self._Header = header(*[index for index, _ in enumerate(hls)])

        self.view = MultiSampleTreeView(self)
        self._view_setup()
        self._assign_delegates()

        super().__init__()
        file_label = self.file_layout.itemAtPosition(0, 0).widget()
        file_label.setText("Add file:")

        self.recent_combo.setLineEdit(RecentLineEdit(self))
        self.recent_combo.setCurrentText("Recent files")
        self.__recent_combo_view_resize()

        open_button = self.file_layout.itemAtPosition(0, 2).widget()
        open_button.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        open_button.setText("Browse...")
        open_button.setIcon(QIcon())

        self.summary_label.hide()
        self.load_data_button.hide()
        self.controlArea.layout().removeItem(self.file_layout)
        self.controlArea.setSizePolicy(
            QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        main_box = gui.hBox(self.left_side, margin=4)
        left_box = gui.vBox(main_box, spacing=10)
        left_box.layout().addLayout(self.file_layout)
        left_box.layout().addWidget(self.view)
        right_box = gui.vBox(main_box)
        right_box.layout().addWidget(self.controlArea)
        output_box = gui.hBox(left_box)
        box = gui.widgetBox(output_box, True)
        gui.radioButtons(
            box, self, "output_type", self.OUTPUT_TYPES,
            label="Output genes:", orientation=Qt.Horizontal,
            callback=self._output_type_changed
        )
        gui.rubber(output_box)
        cb_box = gui.auto_commit(output_box, self, "auto_commit", "Load Data")
        cb_box.setMaximumWidth(250)
        self.left_side.layout().addWidget(main_box)
        self.read_settings()
        self.view.setFocus()

        self.cells_in_rows_changed.connect(self._update_view_cells_genes)

    def _update_view_cells_genes(self):
        row = self.view.currentIndex().row()
        if row > -1:
            cell_item = self.view.model().item(row, self._Header.cells)
            cell_item.setData(self._data_loader.n_cells, Qt.DisplayRole)
            gene_item = self.view.model().item(row, self._Header.genes)
            gene_item.setData(self._data_loader.n_genes, Qt.DisplayRole)
            self.repaint()

    def _view_clicked(self, index):
        if index.column() == self._Header.remove:
            self.remove_item(index)
        elif index.column() == self._Header.selected:
            self.commit()

    def _selection_changed(self):
        rows = self.view.selectionModel().selectedRows(0)
        current = rows[0] if rows else None  # type: Optional[QModelIndex]
        if current is not None:
            self.set_current_loader(
                current.data(LoaderObjectRole),
                current.sibling(
                    current.row(), self._Header.name
                ).data(Qt.DisplayRole)
            )

    def _output_type_changed(self):
        self.commit()

    def _assign_delegates(self):
        self.view.setItemDelegateForColumn(
            self._Header.name, FileDelegate(self)
        )
        self.view.setItemDelegateForColumn(
            self._Header.cells, NumericalDelegate(self)
        )
        self.view.setItemDelegateForColumn(
            self._Header.genes, NumericalDelegate(self)
        )
        self.view.setItemDelegateForColumn(
            self._Header.size, SizeDelegate(self)
        )
        self.view.setItemDelegateForColumn(
            self._Header.sparsity, SparsityDelegate(self)
        )

    def _view_setup(self):
        model = MultiSampleModel(self)
        model.setHorizontalHeaderLabels(self._header_labels)
        self.view.setModel(model)
        self.view.clicked[QModelIndex].connect(self._view_clicked)
        self.view.data_changed.connect(lambda: self.commit())
        self.view.drop_finished.connect(lambda: self.commit())
        self.view.header().setStretchLastSection(False)
        self.view.header().setSectionResizeMode(
            self._Header.name, QHeaderView.Stretch
        )
        self.view.selectionModel().selectionChanged.connect(
            self._selection_changed
        )
        self.view.resizeColumnToContents(self._Header.selected)
        self.view.setColumnWidth(self._Header.remove, 20)
        self._resize_columns_to_contents()

    def _resize_columns_to_contents(self):
        self.view.resizeColumnToContents(self._Header.cells)
        self.view.resizeColumnToContents(self._Header.genes)
        self.view.resizeColumnToContents(self._Header.size)
        self.view.resizeColumnToContents(self._Header.sparsity)

    def add_item(self, checked, source_name):
        path = self.current_path()
        if not path:
            return

        self.Information.file_in_list.clear()
        if len(self.view.model().findItems(path, Qt.MatchFixedString, 1)):
            self.Information.file_in_list(path)
            return

        exists = os.path.exists(path)
        self.view.model().appendRow([
            self.__selected_item(checked, exists),
            self.__file_item(exists),
            self.__source_item(source_name),
            self.__cells_item(),
            self.__genes_item(),
            self.__size_item(),
            self.__sparsity_item(),
            self.__remove_item()
        ])
        self._resize_columns_to_contents()
        self.commit()

    def __selected_item(self, checked, exists):
        item = QStandardItem()
        item.setData(self._data_loader, LoaderObjectRole)
        item.setCheckable(exists)
        item.setCheckState(Qt.Checked if checked and exists else Qt.Unchecked)
        item.setEditable(False)
        return item

    def __file_item(self, exists):
        item = QStandardItem()
        item.setData(self.current_path(), Qt.DisplayRole)
        item.setToolTip(self.current_path())
        item.setEditable(False)
        if not exists:
            item.setToolTip("Missing from file system")
            item.setForeground(QBrush(QColor(Qt.red)))
        return item

    def __source_item(self, source):
        return QStandardItem(source or os.path.split(self.current_path())[1])

    def __cells_item(self):
        item = QStandardItem()
        item.setData(self._data_loader.n_cells, Qt.DisplayRole)
        item.setEditable(False)
        return item

    def __genes_item(self):
        item = QStandardItem()
        item.setData(self._data_loader.n_genes, Qt.DisplayRole)
        item.setEditable(False)
        return item

    def __size_item(self):
        item = QStandardItem()
        item.setData(self._data_loader.file_size, Qt.DisplayRole)
        item.setEditable(False)
        return item

    def __sparsity_item(self):
        item = QStandardItem()
        item.setData(self._data_loader.sparsity, Qt.DisplayRole)
        item.setEditable(False)
        return item

    def __remove_item(self):
        item = QStandardItem()
        item.setIcon(
            self.style().standardIcon(QStyle.SP_DockWidgetCloseButton))
        item.setEditable(False)
        return item

    def select_item(self, index):
        if not isinstance(index, QModelIndex):
            index = self.view.model().index(index, 0)
        self.view.selectionModel().setCurrentIndex(
            index,
            QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows)
        self.view.setFocus()
        self.repaint()

    def remove_item(self, index):
        self.view.model().removeRows(index.row(), 1)
        self.commit()
        if not self.view.model().rowCount():
            self.set_current_loader(Loader(), "")

    def select_last_item(self):
        n_rows = self.view.model().rowCount()
        if n_rows == 0:
            return
        n_cols = self.view.model().columnCount()
        self.select_item(self.view.model().index(n_rows - 1, n_cols - 1))

    def set_current_loader(self, loader, path):
        self._data_loader = loader
        self._current_path = path
        self.setup_gui()

    def set_current_path(self, path, checked=True, source_name=""):
        super().set_current_path(path)
        self.add_item(checked, source_name)
        self.recent_combo.setCurrentText("Recent files")

    def _select_recent(self, index):
        super()._select_recent(index)
        self.select_last_item()

    @Slot()
    def browse(self):
        dlg = QFileDialog(
            self, acceptMode=QFileDialog.AcceptOpen,
            fileMode=QFileDialog.ExistingFiles,
        )
        filters = owloaddata.Formats
        dlg.setNameFilters(filters)
        if filters:
            dlg.selectNameFilter(filters[-1])
        if dlg.exec_() == QFileDialog.Accepted:
            for filename in dlg.selectedFiles():
                self.set_current_path(filename)
                self.select_last_item()
        self.__recent_combo_view_resize()

    def __recent_combo_view_resize(self):
        self.recent_combo.view().setMinimumHeight(
            self.recent_combo.view().sizeHintForRow(0) *
            self.recent_model.rowCount())

    def _invalidate(self):
        self.commit()

    def commit(self):
        if not self.view or not self.view.model():
            return

        self.clear_messages()
        self.write_settings()

        data_collection = []
        for index, (path, checked, source_name) in enumerate(self.samples):
            if checked:
                loader = self.loaders.get(path)
                table = loader()
                if any(loader.errors.values()):
                    self.select_item(index)
                    self.show_error_messages()
                    break
                else:
                    data_collection.append((table, source_name))

        data = Concatenate.concatenate(self.output_type, data_collection)
        self.Outputs.data.send(data)

    def write_settings(self):
        self.samples = []
        self.loaders = {}
        model = self.view.model()
        for i in range(model.rowCount()):
            item = model.item(i)
            checked, loader = item.checkState(), item.data(LoaderObjectRole)
            path = model.item(i, self._Header.name).text()
            source_name = model.item(i, self._Header.source).text()
            self.samples.append((path, checked, source_name))
            loader.recent_path = self.relocate_path(loader.recent_path)
            loader.row_annotation_file = self.relocate_path(
                loader.row_annotation_file)
            loader.col_annotation_file = self.relocate_path(
                loader.col_annotation_file)
            self.loaders[path] = loader

    def saveSettings(self):
        self.write_settings()
        super().saveSettings()

    def _saveState(self):
        super()._saveState()
        self.write_settings()

    def read_settings(self):
        samples = self.samples.copy()
        loaders = self.loaders.copy()
        for path, checked, source_name in samples:
            self._current_path = path
            loader = loaders.get(path)
            if loader is not None:
                loader = loader.copy()
                resolved_path = self.resolve_path(loader.recent_path)
                loader.recent_path = resolved_path
                loader.row_annotation_file = self.resolve_path(
                    loader.row_annotation_file)
                loader.col_annotation_file = self.resolve_path(
                    loader.col_annotation_file)
                self._current_path = resolved_path.abspath
                self.set_current_loader(loader, resolved_path.abspath)
                self.add_item(checked, source_name)
        self.select_last_item()
        self.recent_combo.setCurrentText("Recent files")

    def send_report(self):
        model = self.view.model()
        paths = [os.path.split(model.item(i, self._Header.name).text())[1] for
                 i in range(model.rowCount()) if model.item(i).checkState()]
        self.report_items(
            "Concatenated samples",
            [("Files", ", ".join(paths) if paths else "No files."),
             ("Output genes", self.OUTPUT_TYPES[self.output_type])])


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    app = QApplication([])
    w = OWMultiSample()
    w.show()
    w.raise_()
    app.exec_()
    w.saveSettings()
    w.onDeleteWidget()



import os
import unittest
from unittest.mock import patch, Mock
from pathlib import PurePath

import numpy.testing as npt

from AnyQt.QtCore import Qt, QMimeData, QUrl, QPoint
from AnyQt.QtGui import QDropEvent

from Orange.widgets.tests.base import WidgetTest

from orangecontrib.single_cell.widgets.owmultisample import OWMultiSample


class TestOWMultiSample(WidgetTest):
    def create_widget(self, cls, stored_settings=None,
                      reset_default_settings=True):
        if reset_default_settings:
            self.reset_default_settings(cls)
        widget = cls.__new__(cls, signal_manager=self.signal_manager,
                             stored_settings=stored_settings)
        widget.__init__()
        self.process_events()
        self.widgets.append(widget)
        return widget

    def setUp(self):
        self.widget = self.create_widget(
            OWMultiSample, stored_settings={"auto_commit": True}
        )
        self._path = path = os.path.join(os.path.dirname(__file__), "data")
        self.file_name_1 = os.path.join(path, "10x/hg19/matrix.mtx")
        self.file_name_2 = os.path.join(path, "10x/mm10/matrix.mtx")
        self.widget.set_current_path(self.file_name_1)
        self.widget.set_current_path(self.file_name_2)
        model = self.widget.view.model()
        model.item(0).setCheckState(True)
        model.item(1).setCheckState(True)
        self.widget.commit()

    def test_minimum_size(self):
        pass

    def test_load_samples(self):
        self.assertEqual(self.widget.view.model().rowCount(), 2)

    def test_concatenate_intersection_mtx(self):
        concatenated_data = self.get_output("Data")
        domain = concatenated_data.domain
        self.assertEqual(len(concatenated_data), 11)
        self.assertEqual(len(domain.attributes), 1)
        self.assertEqual(len(domain.metas), 2)

    def test_concatenate_union_mtx(self):
        self.widget.controls.output_type.buttons[1].click()
        concatenated_data = self.get_output("Data")
        domain = concatenated_data.domain
        self.assertEqual(len(concatenated_data), 11)
        self.assertEqual(len(domain.attributes), 8)
        self.assertEqual(len(domain.metas), 2)
        self.assertTrue(all(
            [list(attr.attributes.keys()) == ["Id", "Gene"]
             for attr in domain.attributes]
        ))

    def test_settings(self):
        self.widget.saveSettings()
        widget = self.create_widget(
            OWMultiSample, reset_default_settings=False
        )
        self.assertEqual(widget.view.model().rowCount(), 2)
        self.assertIsNotNone(self.get_output("Data", widget))
        npt.assert_array_equal(
            self.get_output("Data").X, self.get_output("Data", widget).X
        )

    def test_drop_sample(self):
        path = os.path.join(self._path, "lib.cell.count")
        data = QMimeData()
        data.setUrls([QUrl.fromLocalFile(path)])
        event = QDropEvent(
            QPoint(10, 10), Qt.MoveAction, data,
            Qt.NoButton, Qt.NoModifier, QDropEvent.Drop)
        self.widget.view.dropEvent(event)
        self.assertEqual(self.widget.view.model().rowCount(), 3)

    @patch("Orange.widgets.widget.OWWidget.workflowEnv",
           Mock(return_value={"basedir": os.path.join(os.path.dirname(__file__), "data")}))
    def test_load_workflow(self):
        base_path = os.path.join("010-Showcase-LoadDataM-Data", "batch-A")
        base_name = str(PurePath(os.path.join(base_path, "matrix.mtx")))
        base_row_name = str(PurePath(os.path.join(base_path, "barcodes.tsv")))
        base_col_name = str(PurePath(os.path.join(base_path, "genes.tsv")))

        # store settings - simulate saving workflow
        w1 = self.create_widget(OWMultiSample)
        w1.set_current_path(os.path.join(os.path.join(os.path.dirname(__file__), "data"), str(base_name)))
        w1.write_settings()
        settings = self.widget.settingsHandler.pack_data(w1)

        # check other widget settings - simulate opening workflow
        w2 = self.create_widget(OWMultiSample, stored_settings=settings)
        loader = w2._data_loader

        self.assertEqual(str(PurePath(w2._recent[0].relpath)), base_name)
        self.assertEqual(w2._recent[0].prefix, "basedir")
        self.assertEqual(str(PurePath(loader.recent_path.relpath)), base_name)
        self.assertEqual(loader.recent_path.prefix, "basedir")
        self.assertEqual(str(PurePath(loader.row_annotation_file.relpath)), base_row_name)
        self.assertEqual(loader.row_annotation_file.prefix, "basedir")
        self.assertEqual(str(PurePath(loader.col_annotation_file.relpath)), base_col_name)
        self.assertEqual(loader.col_annotation_file.prefix, "basedir")


if __name__ == "__main__":
    unittest.main()
